import pygame

pygame.font.init()


def get_all_system_fonts():
    """
    返回所有可用的系统字体名称
    :return: list=[str,str,str...]
    """
    fonts = pygame.font.get_fonts()
    return fonts


class SqrtSFont:
    def __init__(self, font_size, font_name, font_theme):
        self.font_size = font_size
        self.font_name = font_name
        self.font = pygame.font.Font(font_theme.get_fonts()[font_name], font_size)

    def render_text(self, text, color, antialias=True, background=None):
        """
        渲染文字，返回文字
        :param text:文字str
        :param color:文字颜色
        :param antialias:是否抗锯齿
        :param background:背景颜色（int,int,int）
        :return:surface
        """
        return self.font.render(text, antialias, color, background)


class SystemFont:
    def __init__(self, font_size, font_name):
        self.font_size = font_size
        self.font_name = font_name
        self.font = pygame.font.SysFont(font_name, font_size)

    def render_text(self, text, color, antialias=True, background=None):
        """
        渲染文字，返回文字
        :param text:文字str
        :param color:文字颜色
        :param antialias:是否抗锯齿
        :param background:背景颜色（int,int,int）
        :return:surface
        """
        return self.font.render(text, antialias, color, background)



import win32con
import win32gui  # 界面


def get_hwnd(name):
    """
    获取窗口权柄
    :param name: 窗口名称
    :return: 窗口权柄int
    """
    ck = 0

    def get_all_hwnd(hwnd, mouse):
        nonlocal ck
        hwnd_title = dict()
        if win32gui.IsWindow(hwnd) and win32gui.IsWindowEnabled(hwnd) and win32gui.IsWindowVisible(hwnd):
            hwnd_title.update({hwnd: win32gui.GetWindowText(hwnd)})
        for h, t in hwnd_title.items():
            if t == name:
                ck = h

    win32gui.EnumWindows(get_all_hwnd, 0)

    return ck


def _get_window_pos(hwnd):
    """
    获得窗口坐标
    :param hwnd:窗口权柄
    :return: pos_list
    """
    x, y, _, _ = win32gui.GetWindowRect(hwnd)
    return [x, y]


class WindowMover:
    """
    窗口移动器
    """
    def __init__(self):
        """
        None_param
        """
        self.title = None
        self.size = None
        self.ck = None

    def update(self, window):
        """
        更新坐标，内部函数
        :param window: Window
        :return: None
        """
        self.title = window.title
        self.size = window.get_real_window_size()
        self.ck = get_hwnd(self.title)

    def moveto(self, new_pos):
        """
        移动到坐标
        :param new_pos:[int,int]
        :return: None
        """
        win32gui.SetWindowPos(self.ck,
                              win32con.HWND_TOPMOST,  # 设置的窗口位置，最上面
                              new_pos[0],
                              new_pos[1],  # y坐标
                              self.size[0],  # 窗口长度
                              self.size[1],  # 窗口宽度
                              win32con.SWP_SHOWWINDOW  # 显示窗口
                              )

    def get_window_pos(self):
        """
        获取窗口坐标
        :return: pos
        """
        return _get_window_pos(self.ck)


if __name__ == "__main__":
    ...



from SqrtS.Layouts.Layout import Layout
from SqrtS.Widgets.Widget import Widget


class FloatingLayout(Layout):
    """
    浮动布局
    """

    def __init__(self, name):
        """

        :param name:布局名
        """
        super().__init__(name=name)
        self.widgets_poses = {}
        self.widgets = []

    def compute_poses(self):
        """
        计算更新坐标
        :return: None
        """
        for i in self.widgets_poses.keys():
            for ii in self.widgets:
                if ii.IDname == i:
                    ii.bind_pos(self.widgets_poses[i])

    def bind_widgets_poses(self, widgets_pos_dict: dict):
        """
        绑定每个控件的坐标
        :param widgets_pos_dict: 空间坐标字典,{"IDname?":[int,int],...}
        :return: None
        """
        for key in widgets_pos_dict.keys():
            self.widgets_poses[str(key)] = widgets_pos_dict[key]

        self.compute_poses()

    def blit_all(self, window):
        """
        在屏幕上画出所有的控件
        :return: None
        """
        for i in self.widgets:
            if i.widget_alive:
                i.blit(window)
            else:
                self.widgets.remove(i)

    def append(self, weight: Widget):
        """
        添加控件
        :param weight:Widget
        :return: None
        """
        self.widgets.append(weight)
        self.compute_poses()

    def appends(self, weights: list):
        """
        批量添加控件（推荐）
        :param weights: []
        :return: None
        """
        for i in weights:
            self.widgets.append(i)
        self.compute_poses()

    def adds(self, weights):
        """
        追加已经由外部设定好坐标的控件
        :param weights: list
        :return: None
        """
        for i in weights:
            self.widgets.append(i)



class AnimationSystem:
    def __init__(self):
        self.animations = []
        self.dynamic_effects = []

    def update_blit_animations(self, window):
        """
        更新动画效果
        :param window:Window
        :return: None
        """
        for i in self.animations:
            i.blit_flip(window)
        self.update_dynamic_effects(window)

    def update_dynamic_effects(self, window):
        for i in self.dynamic_effects:
            i.update(window)

    def register_for_animation(self, animation):
        """
        注册动画，注册开始之日便是运行之时
        :param animation: 动画对象
        :return: None
        """
        self.animations.append(animation)

    def register_for_dynamic_effects(self, animation):
        """
        注册动态效果
        :param animation:效果
        :return:None
        """
        self.dynamic_effects.append(animation)

    def destroy_animation(self, animation):
        """

        :param animation: 动画对象
        :return: None
        """
        for i in self.animations:
            if i == animation:
                self.animations.remove(i)

    def destroy_dynamic_effects(self, animation):
        """

        :param animation: 动效对象
        :return: None
        """
        for i in self.dynamic_effects:
            if i == animation:
                self.dynamic_effects.remove(i)



from PIL import Image
import os
from SqrtS.Animation.LoopAnimation import LowerLoopAnimation


class GIFSplit:

    def __init__(self, file_name):
        self.file_name = file_name  # 传入的文件名
        self.dir_name = self.file_name[:-4]  # 根据文件名创建存放分帧图片的文件夹
        self.gif_path = os.path.join(os.path.dirname(__file__), file_name)  # 拼接图片文件的完整路径（仅限同一文件夹内）
        self.make_dir()

        self.file_exist = False

    def make_dir(self):
        """用于创建存放分帧图片的文件夹"""
        try:
            os.mkdir(self.dir_name)
        except FileExistsError as e:
            self.file_exist = True

    def framing_test(self):
        """GIF图片分帧"""
        if not self.file_exist:
            img = Image.open(self.gif_path)
            try:
                while True:
                    curr = img.tell()
                    name = os.path.join(self.dir_name, '%s.png' % str(curr))
                    img.save(name)
                    img.seek(curr + 1)
            except Exception as e:
                pass

    def get_avg_fps(self):
        """
        获取PIL GIF的频率
        :return: None
        """
        pio = Image.open(self.file_name)
        pio.seek(0)
        frames = duration = 0
        while True:
            try:
                frames += 1
                duration += pio.info['duration']
                pio.seek(pio.tell() + 1)
            except EOFError:
                return int(frames / duration * 1000)


class GIFAnimation:
    def __init__(self, gif_path, IDname, size=None):
        self.gif_split = GIFSplit(gif_path)
        self.gif_split.framing_test()
        self.fps = 1 / self.gif_split.get_avg_fps()
        self.IDname = IDname
        self.animation = LowerLoopAnimation(IDname=IDname)
        self.animation.load_animation(self.gif_split.dir_name, size=size)

    def play(self, window, pos):
        self.animation.play(window, pos, self.fps)

    def stop(self, window):
        self.animation.stop(window)

    def pause(self, window):
        self.animation.pause(window)

    def unpause(self, window, pos):
        self.animation.unpause(window, pos, self.fps)


if __name__ == '__main__':
    gf = GIFSplit('C:\\Users\\aa\\PycharmProjects\\SqrtSGUI\\sources\\target2.gif')
    print(gf.get_avg_fps())



import pygame


class ImageSurface:
    def __init__(self, img_path):
        """
        图片类，近似于数据类，功能较少
        :param img_path:
        """
        self.img_surface = pygame.image.load(img_path)

    def resize(self, new_size):
        """
        对当前图片进行重新设置大小
        :param new_size: 大小列表
        :return: None
        """
        return pygame.transform.scale(self.img_surface, new_size)

    def return_surface(self):
        """
        返回图片对象
        :return: None
        """
        return self.img_surface

    def reload(self, new_img_path):
        """
        重新导入图片，更新刷新图片
        :param new_img_path:
        :return:
        """
        self.img_surface = pygame.image.load(new_img_path)


class SurfaceImage:
    def __init__(self, surface):
        """
        图片类，近似于数据类，功能较少
        :param img_path:
        """
        self.img_surface = surface

    def resize(self, new_size):
        """
        对当前图片进行重新设置大小
        :param new_size: 大小列表
        :return: None
        """
        return pygame.transform.scale(self.img_surface, new_size)

    def return_surface(self):
        """
        返回图片对象
        :return: None
        """
        return self.img_surface

    def reload(self, new_img_path):
        """
        重新导入图片，更新刷新图片
        :param new_img_path:
        :return:
        """
        self.img_surface = pygame.image.load(new_img_path)



import keyboard
import threading


class GlobalKeyBoard:
    def __init__(self):
        self.hotkeys = {}

    def add(self, hot_key, func):
        self.hotkeys[hot_key] = func

    def run(self):
        def run_threading(*args):
            for key in self.hotkeys:
                keyboard.add_hotkey(key, self.hotkeys[key])
            keyboard.wait()

        threading.Thread(target=run_threading).start()



KEYDOWN = 0
KEYUP = 1
MOUSEDOWN = 2
MOUSEUP = 3
EVERYTIME = 4
DROPFILE = 5
TEXTINPUT = 6

MOUSE_BUTTON_LEFT_DOWN = 7
MOUSE_MID_DOWN = 8
MOUSE_RIGHT_DOWN = 9

MOUSE_BUTTON_LEFT_UP = 10
MOUSE_MID_UP = 11
MOUSE_RIGHT_UP = 12

MOUSE_WHEEL_UP = 13
MOUSE_WHEEL_DOWN = 14


class EventProcessor:
    def __init__(self):
        self.key_down_pool = []
        self.key_up_pool = []
        self.mouse_up_pool = []
        self.mouse_down_pool = []
        self.everytime_pool = []
        self.dropfile_pool = []
        self.input_pool = []

        self.mouse_left_down_pool = []
        self.mouse_left_up_pool = []

        self.mouse_mid_down_pool = []
        self.mouse_mid_up_pool = []

        self.mouse_right_down_pool = []
        self.mouse_right_up_pool = []

        self.mouse_wheel_down_pool = []
        self.mouse_wheel_up_pool = []

    def key_down_process(self, event, window):
        """
        刷新按下按键池
        :param window: Window
        :param event: 事件
        :return: None
        """
        for i in self.key_down_pool:
            i(event, window)

    def key_up_process(self, event, window):
        """
        刷新抬起按键池
        :param window: Window
        :param event: 事件
        :return: None
        """
        for i in self.key_up_pool:
            i(event, window)

    def mouse_down(self, event, window):
        """
        刷新按下鼠标池
        :param window: Window
        :param event: 事件
        :return: None
        """
        for i in self.mouse_down_pool:
            i(event, window)

    def mouse_up(self, event, window):
        """
        刷新抬起鼠标池
        :param window: Window
        :param event: 事件
        :return: None
        """
        for i in self.mouse_up_pool:
            i(event, window)

    def everytime_event(self, window):
        """
        刷新周期事件池
        :return: None
        """
        for i in self.everytime_pool:
            i(window)

    def drop_file_event(self, event, window):
        """
        拖拽文件事件池
        :param event: 事件
        :param window: 窗口
        :return:
        """
        for i in self.dropfile_pool:
            i(event, window)

    def text_input_event(self, event, window):
        """
        文字输入池，支持多种文字
        :param event: 事件
        :param window: 窗口
        :return: None
        """
        for i in self.input_pool:
            i(event, window)

    def register_event(self, type_, function):
        """
        注册事件，所有被注册函数都需要有一个参数：event
        :param type_: 注册类型
        :param function: 函数体
        :return: None
        """
        if type_ == KEYDOWN:
            self.key_down_pool.append(function)
        elif type_ == KEYUP:
            self.key_up_pool.append(function)
        elif type_ == MOUSEDOWN:
            self.mouse_down_pool.append(function)
        elif type_ == MOUSEUP:
            self.mouse_up_pool.append(function)
        elif type_ == EVERYTIME:
            self.everytime_pool.append(function)
        elif type_ == DROPFILE:
            self.dropfile_pool.append(function)
        elif type_ == TEXTINPUT:
            self.input_pool.append(function)

        elif type_ == MOUSE_BUTTON_LEFT_DOWN:
            self.mouse_left_down_pool.append(function)
        elif type_ == MOUSE_BUTTON_LEFT_UP:
            self.mouse_left_up_pool.append(function)

        elif type_ == MOUSE_RIGHT_DOWN:
            self.mouse_right_down_pool.append(function)
        elif type_ == MOUSE_RIGHT_UP:
            self.mouse_right_up_pool.append(function)

        elif type_ == MOUSE_MID_DOWN:
            self.mouse_mid_down_pool.append(function)
        elif type_ == MOUSE_MID_UP:
            self.mouse_mid_up_pool.append(function)

        elif type_ == MOUSE_WHEEL_DOWN:
            self.mouse_wheel_down_pool.append(function)
        elif type_ == MOUSE_WHEEL_UP:
            self.mouse_wheel_up_pool.append(function)

    def destroy_event(self, type_, function):
        """
        注销事件，传进函数体
        :param type_: 注册类型
        :param function: 函数体
        :return: None
        """
        if type_ == KEYDOWN:
            for i in self.key_down_pool:
                if i == function:
                    self.key_down_pool.remove(i)

        elif type_ == KEYUP:
            for i in self.key_up_pool:
                if i == function:
                    self.key_up_pool.remove(i)

        elif type_ == MOUSEDOWN:
            for i in self.mouse_down_pool:
                if i == function:
                    self.mouse_down_pool.remove(i)

        elif type_ == MOUSEUP:
            for i in self.mouse_up_pool:
                if i == function:
                    self.mouse_up_pool.remove(i)

        elif type_ == EVERYTIME:
            for i in self.everytime_pool:
                if i == function:
                    self.everytime_pool.remove(i)

        elif type_ == DROPFILE:
            for i in self.dropfile_pool:
                if i == function:
                    self.dropfile_pool.remove(i)

        elif type_ == TEXTINPUT:
            for i in self.input_pool:
                if i == function:
                    self.input_pool.remove(i)

        elif type_ == MOUSE_BUTTON_LEFT_DOWN:
            for i in self.mouse_left_down_pool:
                if i == function:
                    self.mouse_left_down_pool.remove(i)
        elif type_ == MOUSE_BUTTON_LEFT_UP:
            for i in self.mouse_left_up_pool:
                if i == function:
                    self.mouse_left_up_pool.remove(i)
        elif type_ == MOUSE_RIGHT_DOWN:
            for i in self.mouse_right_down_pool:
                if i == function:
                    self.mouse_right_down_pool.remove(i)
        elif type_ == MOUSE_RIGHT_UP:
            for i in self.mouse_right_down_pool:
                if i == function:
                    self.mouse_right_up_pool.remove(i)
        elif type_ == MOUSE_MID_DOWN:
            for i in self.mouse_mid_down_pool:
                if i == function:
                    self.mouse_mid_down_pool.remove(i)
        elif type_ == MOUSE_MID_UP:
            for i in self.mouse_mid_up_pool:
                if i == function:
                    self.mouse_mid_up_pool.remove(i)

        elif type_ == MOUSE_WHEEL_DOWN:
            for i in self.mouse_wheel_down_pool:
                if i == function:
                    self.mouse_wheel_down_pool.remove(i)
        elif type_ == MOUSE_WHEEL_UP:
            for i in self.mouse_wheel_up_pool:
                if i == function:
                    self.mouse_wheel_up_pool.remove(i)

    def mouse_left_down(self, event, window):
        """
        刷新按下鼠标池
        :param window: Window
        :param event: 事件
        :return: None
        """
        for i in self.mouse_left_down_pool:
            i(event, window)

    def mouse_left_up(self, event, window):
        """
        刷新抬起鼠标池
        :param window: Window
        :param event: 事件
        :return: None
        """
        for i in self.mouse_left_up_pool:
            i(event, window)

    def mouse_mid_down(self, event, window):
        """
        刷新按下鼠标池
        :param window: Window
        :param event: 事件
        :return: None
        """
        for i in self.mouse_mid_down_pool:
            i(event, window)

    def mouse_mid_up(self, event, window):
        """
        刷新抬起鼠标池
        :param window: Window
        :param event: 事件
        :return: None
        """
        for i in self.mouse_mid_up_pool:
            i(event, window)

    def mouse_right_down(self, event, window):
        """
        刷新按下鼠标池
        :param window: Window
        :param event: 事件
        :return: None
        """
        for i in self.mouse_right_down_pool:
            i(event, window)

    def mouse_right_up(self, event, window):
        """
        刷新抬起鼠标池
        :param window: Window
        :param event: 事件
        :return: None
        """
        for i in self.mouse_right_up_pool:
            i(event, window)

    def mouse_wheel_down(self, event, window):
        """
        刷新按下鼠标池
        :param window: Window
        :param event: 事件
        :return: None
        """
        for i in self.mouse_wheel_down_pool:
            i(event, window)

    def mouse_wheel_up(self, event, window):
        """
        刷新抬起鼠标池
        :param window: Window
        :param event: 事件
        :return: None
        """
        for i in self.mouse_wheel_up_pool:
            i(event, window)



from SqrtS.Widgets.Widget import Widget
from SqrtS.Errors.widgets_error import BindPosesError
import pygame


class FrameBox(Widget):
    def __init__(self, IDname, size, background_image_path=""):
        super().__init__([0, 0], IDname)
        self.size = size

        if len(background_image_path):
            self.background_sur = pygame.transform.scale(pygame.image.load(background_image_path), size)
        else:
            self.background_sur = None

        self.pos = [0, 0]
        self.offset_pos = {}
        self.widgets = []

    def appends_widgets(self, widgets_list):
        for i in widgets_list:
            self.offset_pos[i.IDname] = []
            self.widgets.append(i)

    def bind_pos(self, new_pos: list):
        self.pos = new_pos
        try:
            for i in self.widgets:
                i.bind_pos([self.pos[0] + self.offset_pos[i.IDname][0], self.pos[1] + self.offset_pos[i.IDname][1]])
        except IndexError as e:
            raise BindPosesError("绑定坐标错误！")

    def blit(self, window):
        if self.background_sur:
            window.get_window().blit(self.background_sur, self.pos)
        for i in self.widgets:
            i.blit(window)

    def register_for_event(self, window):
        for i in self.widgets:
            i.register_for_event(window)

    def bind_widgets_poses(self, pos_dict):
        """
        为framebox内的控件绑定坐标
        :param pos_dict:
        :return:
        """
        for key in pos_dict.keys():
            for i in self.widgets:
                if i.IDname == key:
                    i.bind_pos([self.pos[0] + pos_dict[key][0], self.pos[1] + pos_dict[key][1]])
                    self.offset_pos[key] = pos_dict[key]


class LinerFrameBox(FrameBox):
    def __init__(self, IDname, size, direction="longitudinal"):
        """

        :param IDname: str
        :param size: [int,int]
        :param direction: longitudinal(纵向排列) or lateral(横向排列)
        """
        super().__init__(IDname, size)
        self.direction = direction
        self.offset_now = 0
        self.buffer_widgets = []
        self.size = [0, 0]

    def bind_widgets_poses(self, pos_dict):
        ...

    def compute_poses(self):
        if self.direction == "longitudinal":
            for i in self.buffer_widgets:
                self.offset_pos[i.IDname] = []
                self.widgets.append(i)
                i.bind_pos([self.pos[0], self.pos[1] + self.offset_now])
                self.offset_pos[i.IDname] = [self.pos[0], self.offset_now]
                self.offset_now += i.size[1]

        elif self.direction == "lateral":
            for i in self.buffer_widgets:
                self.offset_pos[i.IDname] = []
                self.widgets.append(i)
                i.bind_pos([self.pos[1] + self.offset_now, self.pos[1]])
                self.offset_pos[i.IDname] = [self.offset_now, self.pos[1]]
                self.offset_now += i.size[0]

        max_height = 0
        max_width = 0
        for i in self.widgets:
            if i.size[0] > max_width:
                max_width = i.size[0]
            if i.size[1] > max_height:
                max_height = i.size[1]

        self.size = [max_width, max_height]

    def appends_widgets(self, widgets_list):
        for i in widgets_list:
            self.buffer_widgets.append(i)

        max_height = 0
        max_width = 0
        for i in self.widgets:
            if i.size[0] > max_width:
                max_width = i.size[0]
            if i.size[1] > max_height:
                max_height = i.size[1]

        self.size = [max_width, max_height]



import pygame
from SqrtS.Widgets.Widget import Widget
from SqrtS.tools.font import SqrtSFont, SystemFont
from SqrtS.Core.event_process import MOUSEUP, MOUSEDOWN, EVERYTIME, MOUSE_BUTTON_LEFT_DOWN, MOUSE_BUTTON_LEFT_UP


class Texts(Widget):
    def __init__(self,
                 text,
                 color,
                 font,
                 IDname: str,
                 if_antialias=True,
                 background=None):
        """

        :param text: str
        :param color: [int,int,int]
        :param font: SqrtSFont
        :param IDname: str
        :param if_antialias:bool
        :param background: [int,int,int]
        """

        self.chosen = False
        self.if_this_time_call_on = False
        self.click = False
        self.move_out_function = None
        self.up_function = None
        self.call_function = None
        self.move_on_function = None

        self.pos = []
        self.text_surface = font.render_text(text, color, if_antialias, background)
        self.size = self.text_surface.get_rect()[2:]
        self.font = font
        self.text = text
        self.color = color
        super().__init__(size=self.size,
                         IDname=IDname)

        self.event_types = [MOUSEDOWN, MOUSEUP, EVERYTIME]

    def change_text(self, new_text, color, font=None, if_antialias=True, background=None):
        """
        重新更改文字和类型
        :param new_text:新的文字
        :param color: 颜色
        :param font: 字体，SqrtSFont类型
        :param if_antialias: 是否抗锯齿（默认为True）
        :param background: 背景颜色三元组，默认为None
        :return: None
        """
        if font is not None:
            self.font = font
        self.text = new_text
        self.text_surface = font.render_text(self.text, color, if_antialias, background)
        self.size = self.text_surface.get_rect()[2:]

    def change_style(self, color, font=None, if_antialias=True, background=None):
        """
        简简单单地改变样式，只是对上面的change_text做了二次封装而已
        :param color: 上
        :param font: 上
        :param if_antialias:上
        :param background: 上
        :return: None
        """
        if font is not None:
            self.font = font
        self.change_text(self.text, color, font=self.font, if_antialias=if_antialias, background=background)

    def bind_pos(self, new_pos: list):
        """
        绑定坐标
        :param new_pos:
        :return:
        """
        self.pos = new_pos

    def blit(self, window):
        """
        画出图像
        :param window:Window
        :return: None
        """
        window.get_window().blit(self.text_surface, self.pos)

    def check_mouse_down(self, event, window):
        """
        检测鼠标按下并且调用回调函数的函数（笑）
        :param window:
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.size[0] and self.pos[1] < mouse_click_pos[1] < \
                self.pos[1] + self.size[1]:
            self.click = True
            if self.call_function:
                self.call_function(self)

    def check_mouse_up(self, event, window):
        """
        检测鼠标抬起并且调用回调函数的函数（笑）
        :param window:
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.size[0] and self.pos[1] < mouse_click_pos[1] < \
                self.pos[1] + self.size[1]:
            self.click = False
            if self.up_function:
                self.up_function(self)

    def check_mouse_left_down(self, event, window):
        """
        检测鼠标按下并且调用回调函数的函数（笑）
        :param window:
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.size[0] and self.pos[1] < mouse_click_pos[1] < \
                self.pos[1] + self.size[1]:
            self.click = True
            if self.call_function:
                self.call_function(self)

    def check_mouse_left_up(self, event, window):
        """
        检测鼠标抬起并且调用回调函数的函数（笑）
        :param window:
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.size[0] and self.pos[1] < mouse_click_pos[1] < \
                self.pos[1] + self.size[1]:
            self.click = False
            if self.up_function:
                self.up_function(self)

    def weight_everytime_event(self, window):
        """
        每时每刻都在运行的函数为everytime函数，按钮内用于处理按钮移动上去的事件
        :return: None
        """
        mouse_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_pos[0] < self.pos[0] + self.size[0] and self.pos[1] < mouse_pos[1] < \
                self.pos[1] + self.size[1] and not self.click:
            self.chosen = True
            if self.move_on_function and not self.if_this_time_call_on:
                self.move_on_function(self)
                self.if_this_time_call_on = True
        elif self.click:
            ...
        elif self.chosen:
            if self.move_out_function:
                self.move_out_function(self)
            self.chosen = False
            self.if_this_time_call_on = False

    def register_for_event(self, window):
        """
        注册函数，将目标函数注册到EventProcessor中
        :param window: window对象
        :return: None
        """
        window.event_processor.register_event(MOUSE_BUTTON_LEFT_DOWN, self.check_mouse_left_down)
        window.event_processor.register_event(MOUSE_BUTTON_LEFT_UP, self.check_mouse_left_up)
        window.event_processor.register_event(EVERYTIME, self.weight_everytime_event)

    def bind_call_func(self, func):
        """
        绑定回调函数
        :param func:函数体
        :return: None
        """
        self.call_function = func

    def bind_up_func(self, func):
        """
        绑定结束函数
        :param func:函数体
        :return: None
        """
        self.up_function = func

    def bind_on_func(self, func):
        """
        绑定鼠标一上去的函数
        :param func:函数体
        :return: None
        """
        self.move_on_function = func

    def bind_out_func(self, func):
        """
        绑定鼠标一出去的函数
        :param func:函数体
        :return: None
        """
        self.move_out_function = func


class LineTexts(Texts):
    def __init__(self,
                 text,
                 color,
                 font: SqrtSFont,
                 line_length: int,
                 IDname: str,
                 if_antialias=True,
                 background=None):
        super().__init__(text=text,
                         color=color,
                         font=font,
                         if_antialias=if_antialias,
                         background=background,
                         IDname=IDname)
        self.background = background
        self.line_length = line_length
        self.text_list = []
        self.text_surface_list = []

        self.split_texts()

    def split_texts(self):
        """
        划分文字的内部函数
        :return: None
        """
        step = self.line_length
        new_list = []
        for i in range(0, len(self.text), step):
            new_list.append(self.text[i:i + step])

        self.text_list = new_list

        new_surface_lst = []
        for o in self.text_list:
            new_surface_lst.append(
                self.font.render_text(o, self.color, background=self.background)
            )
        self.text_surface_list = new_surface_lst

        self.size = [self.text_surface_list[0].get_rect()[2:][0],
                     self.text_surface_list[0].get_rect()[2:][1] * len(self.text_surface_list)]
        self.size = [self.text_surface_list[0].get_rect()[2:][0],
                     self.text_surface_list[0].get_rect()[2:][1] * len(self.text_surface_list)]

    def change_text(self, new_text, color, font=None, if_antialias=True, background=None):
        """
        重新更改文字和类型
        :param new_text:新的文字
        :param color: 颜色
        :param font: 字体，SqrtSFont类型
        :param if_antialias: 是否抗锯齿（默认为True）
        :param background: 背景颜色三元组，默认为None
        :return: None
        """
        if font is not None:
            self.font = font
        self.text = new_text
        self.background = background
        self.color = color
        self.split_texts()
        self.size = [self.text_surface_list[0].get_rect()[2:][0],
                     self.text_surface_list[0].get_rect()[2:][1] * len(self.text_surface_list)]

    def blit(self, window):
        """
        画图
        :param window:
        :return:
        """
        for index, i in enumerate(self.text_surface_list):
            window.get_window().blit(i, [self.pos[0], self.pos[1] + self.text_surface_list[0].get_rect()[3] * index])



import abc
import threading


class DynamicEffects:
    def __init__(self):
        self.if_finished = False

    def times_call(self, time_interval, times_, func):
        """
        内部函数，用于多次调用，内部采用递归实现
        :param time_interval: 时间间隔
        :param times_: 次数
        :param func: 函数体
        :return: None
        """
        times_ = times_
        count = 0

        def _times_call():
            """
            内部函数
            :return:None
            """
            nonlocal count
            count += 1
            func()
            if count < times_:
                t = threading.Timer(time_interval, _times_call)
                t.start()
            else:
                self.if_finished = True

        _times_call()

    @abc.abstractmethod
    def update(self, window):
        ...

    @abc.abstractmethod
    def _update_dynamic_effects(self):
        ...



from SqrtS.Core.event_process import MOUSEDOWN, MOUSEUP, EVERYTIME
from SqrtS.Widgets.Widget import Widget
import pygame


class MouseClickBoard(Widget):
    def __init__(self, size, IDname):
        """

        :param size: [int,int]
        :param IDname: str
        """
        super().__init__(size, IDname)
        self.click_poses = []
        self.up_poses = []
        self.pos = []
        self.event_types = [MOUSEDOWN, MOUSEUP, EVERYTIME]

    def bind_pos(self, new_pos: list):
        """
        绑定坐标
        :param new_pos:[int,int]
        :return: None
        """
        self.pos = new_pos

    def blit(self, window):
        """

        :param window: None
        :return: None
        """
        ...

    def check_mouse_down(self, event, window):
        """
        检测鼠标按下并且调用回调函数的函数（笑）
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.size[0] and self.pos[1] < mouse_click_pos[1] < \
                self.pos[1] + self.size[1]:
            self.click_poses.append(mouse_click_pos)
        else:
            ...

    def check_mouse_up(self, event, window):
        """
        检测鼠标抬起并且调用回调函数的函数（笑）
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.size[0] and self.pos[1] < mouse_click_pos[1] < \
                self.pos[1] + self.size[1]:
            self.up_poses.append(mouse_click_pos)
        else:
            ...

    def register_for_event(self, window):
        """
        注册函数
        :param window:Window
        :return:
        """
        window.event_processor.register_event(MOUSEDOWN, self.check_mouse_down)
        window.event_processor.register_event(MOUSEUP, self.check_mouse_up)
        window.event_processor.register_event(EVERYTIME, self.weight_everytime_event)



from SqrtS.Widgets.Widget import Widget


class Canvas(Widget):
    def __init__(self):
        super().__init__((0, 0), "canvas")
        self.drawings = []
        self.pos = []

    def bind_pos(self, new_pos: list):
        self.pos = new_pos

    def blit(self, window):
        screen = window.get_window()
        for i in self.drawings:
            i.blit(screen)

    def register_for_event(self, window):
        ...

    def append_drawing(self, drawing):
        self.drawings.append(drawing)

    def appends_drawings(self, drawings):
        for i in drawings:
            self.append_drawing(i)

    def clear(self):
        self.drawings = []



class Pager:
    """
    分页器类
    """

    def __init__(self):
        # 布局字典
        self.layouts = {}

    def append_page(self, layout, window):
        """
        添加一个页面...
        :param layout: 布局
        :return: None
        """
        if not window.Layout:
            window.bind_layout(layout)
        self.layouts[layout.name] = layout

    def change_page(self, window, layout_name, reload_function):
        """
        重新更换页面
        :param window:
        :param layout_name:
        :param reload_function:
        :return:
        """
        window._reload_layout(self.layouts[layout_name], reload_function=reload_function)



from abc import abstractmethod
from SqrtS.Core.event_process import *


class Widget:
    """
    所有控件的基础类
    """

    def __init__(self, size, IDname, many_weights=False):
        self.size = size
        self.many_weight = many_weights
        self.IDname = IDname
        self.event_types = []
        self.widget_alive = True

    @abstractmethod
    def bind_pos(self, new_pos: list):
        ...

    @abstractmethod
    def blit(self, window):
        ...

    @abstractmethod
    def register_for_event(self, window):
        ...

    def destroy_for_event(self, window, types_):
        target_func = None
        for type_ in types_:
            if type_ == MOUSEDOWN:
                target_func = self.check_mouse_down
            elif type_ == MOUSEUP:
                target_func = self.check_mouse_up
            elif type_ == KEYDOWN:
                target_func = self.check_key_down
            elif type_ == KEYUP:
                target_func = self.check_key_up

            elif type_ == MOUSE_BUTTON_LEFT_DOWN:
                target_func = self.check_mouse_left_down
            elif type_ == MOUSE_BUTTON_LEFT_UP:
                target_func = self.check_mouse_left_up
            elif type_ == MOUSE_MID_DOWN:
                target_func = self.check_mouse_mid_down
            elif type_ == MOUSE_MID_UP:
                target_func = self.check_mouse_mid_down
            elif type_ == MOUSE_RIGHT_DOWN:
                target_func = self.check_mouse_right_down
            elif type_ == MOUSE_RIGHT_UP:
                target_func = self.check_mouse_right_up

            elif type_ == MOUSE_WHEEL_DOWN:
                target_func = self.check_mouse_wheel_down
            elif type_ == MOUSE_WHEEL_UP:
                target_func = self.check_mouse_wheel_up
            window.event_processor.destroy_event(type_, target_func)

    def destroy_widget(self, window):
        self.destroy_for_event(window, self.event_types)

        self.widget_alive = False

    def check_key_down(self, event, window):
        ...

    def check_key_up(self, event, window):
        ...

    def check_mouse_down(self, event, window):
        ...

    def check_mouse_up(self, event, window):
        ...

    def weight_everytime_event(self, window):
        ...

    def check_drop_file(self, event, window):
        ...

    def check_text_input(self, event, window):
        ...

    def process_many_weight_poses(self):
        ...

    def check_mouse_left_down(self, event, window):
        ...

    def check_mouse_left_up(self, event, window):
        ...

    def check_mouse_right_down(self, event, window):
        ...

    def check_mouse_right_up(self, event, window):
        ...

    def check_mouse_mid_down(self, event, window):
        ...

    def check_mouse_mid_up(self, event, window):
        ...

    def check_mouse_wheel_down(self, event, window):
        ...

    def check_mouse_wheel_up(self, event, window):
        ...



import json

"""
这个文件是干什么的？
你好，我是SqrtSGUI的作者cemeye！我很鼓励你阅读源代码来提升自己的水平的行为！
呃......你说你是来找BUG的？那也没有关系！让我介绍一下这个文件是干什么的的吧！

这个文件内容这么点，我就随便说几句吧！
这个JSONReader没什么巨大的野心，他只是加载最外层的GLOBAL.json文件的。你不知道么？那个文件里面存着一些需要持久化保存的全局参数！

就这么多，祝你玩得开心！
"""


class JsonReader:
    def __init__(self, json_path):
        self.json_path = json_path

    def read_json(self):
        with open(self.json_path, "r", encoding="utf-8") as f:
            strs = f.read()
            dict_json = json.loads(strs)
        return dict_json


if __name__ == "__main__":
    jc = JsonReader()
    print(jc.read_json())



from SqrtS.Animation.Vector import Vector
from SqrtS.Animation.DynamicEffects import DynamicEffects
from SqrtS.Errors.animation_err import DidNotBindWidgetError


class VectorMove(DynamicEffects):
    def __init__(self, vector, time_interval, x_speed=1, y_speed=1):
        super().__init__()
        self.x_speed = x_speed
        self.y_speed = y_speed

        self.vector = vector

        if self.vector[0] != 0:
            self.forward_x = int(self.vector[0] / abs(self.vector[0]))
        else:
            self.forward_x = 0

        if self.vector[1] != 0:
            self.forward_y = int(self.vector[1] / abs(self.vector[1]))
        else:
            self.forward_y = 0

        self.times = max(abs(self.vector[0]), abs(self.vector[1]))
        self.time_interval = time_interval
        self.target_widget = None
        self.if_started = False

    def bind_widget(self, widget):
        self.target_widget = widget

    def _update_dynamic_effects(self):
        pos = self.target_widget.pos
        x = pos[0]
        y = pos[1]
        if self.vector[0] != 0:
            self.vector[0] -= self.forward_x
            x += self.forward_x * self.x_speed
        if self.vector[1] != 0:
            self.vector[1] -= self.forward_y
            y += self.forward_y * self.y_speed

        self.target_widget.bind_pos([x, y])

    def update(self, window):
        if not self.if_started:
            self.if_started = True
            self.times_call(self.time_interval, self.times, self._update_dynamic_effects)
        else:
            if self.if_finished:
                window.destroy_dynamic_effect(self)


class VectorMoveGroup:
    def __init__(self):
        self.group = []
        self.index = 0

    def bind_all_widget(self, widget):
        for i in self.group:
            i.bind_widget(widget)

    def bind_vectors(self, vectors):
        """

        :param vectors:[VectorMove, VectorMove......]
        :return:
        """
        self.group = vectors

    def update(self, window):
        for index, i in enumerate(self.group):
            if not i.if_finished and self.index == index:
                self.index = i
                i.update(window)
            elif not i.if_finished:
                self.index = i
                i.update(window)



class App:
    """
    代码规范类
    """
    def __init__(self):
        self.window = None

    def bind_window(self):
        ...

    def bind_widgets(self):
        ...

    def register_for_event(self):
        ...

    def bind_poses(self):
        ...

    def run(self):
        self.window.run()

    def bind_layouts(self):
        ...

    def bind_pages(self):
        ...

    def bind_fonts(self):
        ...

    def process_widgets(self):
        ...

    def register_for_tasks(self):
        ...

    def bind_call_functions(self):
        ...




import os
import pystray
import threading
from SqrtS.Errors.core_error import StrayHasBeenRunningError
from PIL import Image
from pystray import MenuItem


class SqrtStray:
    def __init__(self, icon_path):
        """
        系统托盘类
        :param icon_path:托盘图标路径
        """
        self.menu = []
        self.icon_image = Image.open(icon_path)
        self.icon = None

        self.name = "SqrtSGUI"
        self.show_text = "SqrtSGUI"
        self.running = False
        self.bind_move_on("SqrtGUI", "SqrtGUI-running")

    def append_menu(self, menu_name, call_func, enabled=True, visible=True, default=True):
        """
        添加单个项
        :param default: 是否是菜单按钮型的项
        :param visible: 是否可见
        :param enabled: 是否可用
        :param menu_name: 项名
        :param call_func: 回调函数
        :return: None
        """
        if not self.running:
            self.menu.append(
                MenuItem(text=menu_name, action=call_func, enabled=enabled, default=default, visible=visible))
        else:
            raise StrayHasBeenRunningError("系统托盘已经正在运行！无法动态添加菜单！")

    def bind_move_on(self, name, show_text):
        """
        绑定移动到上面的函数
        :param name: 图标名
        :param show_text: 展示的文字
        :return: None
        """
        self.icon = pystray.Icon(name, self.icon_image, show_text, self.menu)
        self.name = name
        self.show_text = show_text

    def notify(self, title, text):
        """
        显示一条消息
        :param title:标题
        :param text: 文字
        :return: None
        """
        if self.icon:
            self.icon.notify(title, text)

    def _run(self):
        """
        内部函数，调用icon的run
        :return:
        """
        self.icon.run()

    @staticmethod
    def close():
        """
        关闭函数
        :return:
        """
        os._exit(0)

    def run(self):
        """
        运行
        :return: None
        """
        self.running = True
        thread = threading.Thread(target=self._run)
        thread.start()


if __name__ == "__main__":
    st = SqrtStray("../tools/theme_root/theme/icon.png")
    st.append_menu("菜单一", lambda: st.notify("hello", "this is GUi"))
    st.bind_move_on("SqrtSGUI", "das")
    st.run()
    while True:
        print("hello")



from SqrtS.Animation.VectorMove import VectorMove, Vector
from SqrtS.Widgets.FrameBox import FrameBox
from SqrtS.Core.core import get_screen_pixel
from SqrtS.Widgets.Widget import Widget


class VerticalSideBar(Widget):
    def __init__(self,
                 window_size,
                 height,
                 time_interval,
                 background_image_path,
                 direction="up",
                 y_speed=1):
        """
        侧边栏类
        :param height:侧边栏宽度
        :param time_interval:移动刷新时间间隔（移动一次需要的时间，其实0.01即可）
        :param background_image_path: 背景图片路径
        :param direction: 方向,只有up和down
        :param y_speed: 每一次移动多少？最好不要改，否则会出现奇怪的效果！
        """
        # 获取窗口大小
        super().__init__([window_size[0], height], "sidebar")
        self.screen_size = window_size

        self.widget_framebox = FrameBox("sidebar_framebox", size=[self.screen_size[0], height],
                                        background_image_path=background_image_path)

        self.y_speed = y_speed
        self.direction = direction
        self.height = height
        self.time_interval = time_interval

        if self.direction == "up":
            self.widget_framebox.bind_pos([0, 0])
            self.move_in_vector = VectorMove(Vector(0, +int(abs(self.height) / self.y_speed)), self.time_interval,
                                             y_speed=self.y_speed)
            self.move_out_vector = VectorMove(Vector(0, -int(abs(self.height) / self.y_speed)), self.time_interval,
                                              y_speed=self.y_speed)
            self.move_in_vector.bind_widget(self.widget_framebox)
            self.move_out_vector.bind_widget(self.widget_framebox)
        elif self.direction == "down":
            self.widget_framebox.bind_pos([0, self.screen_size[1]-self.height])
            self.move_in_vector = VectorMove(Vector(0, -int(abs(self.height) / self.y_speed)), self.time_interval,
                                             y_speed=self.y_speed)
            self.move_out_vector = VectorMove(Vector(0, +int(abs(self.height) / self.y_speed)), self.time_interval,
                                              y_speed=self.y_speed)
            self.move_in_vector.bind_widget(self.widget_framebox)
            self.move_out_vector.bind_widget(self.widget_framebox)

        self.if_out = True

    def bind_pos(self, new_pos: list):
        """
        最好不要用！
        :param new_pos:新坐标
        :return: None
        """
        self.widget_framebox.bind_pos(new_pos)

    def blit(self, window):
        self.widget_framebox.blit(window)

    def register_for_event(self, window):
        """
        注册事件
        :param window: Window
        :return: None
        """
        self.widget_framebox.register_for_event(window=window)

    def appends_widgets(self, widgets_list):
        """
        添加控件列表！
        :param widgets_list:控件
        :return:
        """
        self.widget_framebox.appends_widgets(widgets_list=widgets_list)

    def _make_vector_in(self):
        """
        因为vectorMove是一次性的，所以需要重新生成
        :return: 你猜有没有？
        """
        if self.direction == "up":
            self.move_in_vector = VectorMove(Vector(0, -int(abs(self.height) / self.y_speed)), self.time_interval,
                                             y_speed=self.y_speed)
            self.move_in_vector.bind_widget(self.widget_framebox)
        elif self.direction == "down":
            self.move_in_vector = VectorMove(Vector(0, int(abs(self.height) / self.y_speed)), self.time_interval,
                                             y_speed=self.y_speed)
            self.move_in_vector.bind_widget(self.widget_framebox)

    def _make_vector_out(self):
        """
        因为vectorMove是一次性的，所以需要重新生成
        :return: 你猜有没有？
        """
        if self.direction == "up":
            self.move_out_vector = VectorMove(Vector(0, int(abs(self.height) / self.y_speed)), self.time_interval,
                                              y_speed=self.y_speed)
            self.move_out_vector.bind_widget(self.widget_framebox)
        elif self.direction == "down":
            self.move_out_vector = VectorMove(Vector(0, -int(abs(self.height) / self.y_speed)), self.time_interval,
                                              y_speed=self.y_speed)
            self.move_out_vector.bind_widget(self.widget_framebox)

    def move_in(self, window):
        """
        触发向内移动函数
        :param window: Window
        :return: ∅
        """
        if self.if_out:
            self._make_vector_in()
            window.register_for_dynamic_effect(self.move_in_vector)
            self.if_out = False

    def get_if_out(self):
        return self.if_out

    def move_out(self, window):
        """
        触发向外移动函数
        :param window:Window
        :return: ∅
        """
        if not self.if_out:
            self._make_vector_out()
            window.register_for_dynamic_effect(self.move_out_vector)
            self.if_out = True

    def bind_widgets_poses(self, pos_dict):
        self.widget_framebox.bind_widgets_poses(pos_dict)


if __name__ == "__main__":
    # C:\Users\Administrator\PycharmProjects\SqrtSGUI\sources
    ...



import pygame


def draw_circle(screen, color, pos, radius, border_width):
    pygame.draw.circle(screen, color, pos, radius, border_width)


def draw_ellipse(screen, color, rect, border_width):
    pygame.draw.ellipse(screen, color, rect, border_width)


def draw_arc(screen, color, rect, start_angle, stop_angle, border_width):
    pygame.draw.arc(screen, color, rect, start_angle, stop_angle, border_width)





import os
import zipfile
import pygame

"""
这个文件是干什么的？
你好，我是SqrtSGUI的作者cemeye！我很鼓励你阅读源代码来提升自己的水平的行为！
呃......你说你是来找BUG的？那也没有关系！让我介绍一下这个文件是干什么的的吧！

首先定义了一个主题类，这个类的参数会有另一个函数自动创建，所以不用担心~它里面有一个get_theme方法，使用来获取主题的

walk_one函数是用来遍历单层目录的，你可以不用管它

unzip_file，用来解压zip文件。什么？你不知道zip在这个GUI库里有什么用？他其实是用来压缩主题文件的~

load_theme方法，用来加载主题文件，这个是这个文件的核心函数，调用就调用它。他会返回一个主题类，当然你直接输出他的返回值，你会看到一个字典，但他的返回值
可不是字典哦！这是类Theme的魔术方法“__str__(self),所以返回值是一个类！”

就这么多，祝你玩得开心！
"""


class Theme:
    """
    主题类
    """

    def __init__(self, theme_path):
        pygame.font.init()
        """
        初始化主题
        :param theme_path:主题文件路径
        """
        self.theme_path = theme_path

        self.img_dict = {}

        self.fonts = {}

        self.load()

        self.abs_path = ""

    def __str__(self):
        """
        魔术方法，print输出时会输出当前图片字典
        :return: self.img_dict
        """
        return str(self.img_dict)

    def load(self):
        """
        导入主题文件，原理是扫描目标文件夹后缀为一般图片文件的图片并转换为Surface对象
        :return: None
        """
        sport_imgs = [
            "png", "jpg", "jpeg", "PNG", "JPG", "JPEG"]

        for filename in walk_one(self.theme_path):
            # 检测当前文件是否是主题文件
            if filename.split(".")[-1] in sport_imgs:
                image_name = filename.split(".")[0]
                self.img_dict[image_name] = pygame.image.load(self.theme_path + "\\" + filename)
            else:
                if filename == "fonts":
                    for ii in walk_one(self.theme_path + "\\fonts\\"):
                        if ii.split(".")[-1] in ["ttf", "otf"]:
                            self.fonts[ii.split(".")[0]] = self.theme_path + "fonts\\" + ii

    def get_theme(self):
        """
        获取主题文件，请在load方法后使用！
        :return: self.img_dict / None
        """
        if self.img_dict:
            return self.img_dict
        else:
            print("<#>错误！获取主题失败！")
            return None

    def get_fonts(self):
        """
        获取字体文件
        :return: FONT
        """
        return self.fonts


def walk_one(path):  # 便利单层文件夹
    file_names = []
    for filename in os.listdir(path):
        file_names.append(filename)
    return file_names


def unzip_file(zip_src, dst_dir) -> bool:
    """
    解压zip文件
    :param zip_src: 是zip文件的路径
    :param dst_dir: 是要解压到的目的文件夹
    :return: None
    """
    r = zipfile.is_zipfile(zip_src)
    if r:
        fz = zipfile.ZipFile(zip_src, 'r')
        for file in fz.namelist():
            fz.extract(file, dst_dir)
        return True
    else:
        return False


def load_theme(theme_path):
    """
    导入主题文件
    注意，文件路径请用\\划分，不要用/划分
    :param theme_path: 主题文件
    :return: Theme类
    """
    theme_name = theme_path.split("\\")[-1].split(".")[0]
    theme_length = len(theme_name)
    if not os.path.exists(theme_path.split(".")[0]):
        unzip_file(theme_path, "\\".join(theme_path.split("\\")[0:-1]))

    theme_path_new = theme_path.split("\\")[:-1]
    last_path = ""
    for i in theme_path_new:
        last_path = last_path + i + "\\"
    theme = Theme(last_path + theme_name)
    return theme




"""
hello!  :)

你又来看代码了！这个文件主要定义了一大堆错误，其实每个定义都很简单！
想看具体的错误类型的就翻一下这个文件吧！
"""


class WidgetNotFound(Exception):
    """
    这个错误主要存在于Window.get_widget_by_IDname这个函数！
    如果IDname输错了或者根本没有这个控件就会报这个错误
    """
    ...


class StrayHasBeenRunningError(Exception):
    """
    这个错误主要存在于系统托盘的调用中，当托盘线程启动时，无法动态添加选项！
    """



import pygame


class Line:
    def __init__(self, color, start_pos, end_pos, border_width):
        self.color = color
        self.start_pos = start_pos
        self.end_pos = end_pos
        self.border_width = border_width

    def blit(self, screen):
        pygame.draw.line(screen, self.color, self.start_pos, self.end_pos, self.border_width)


class Lines:
    def __init__(self, color, if_closed, border_width, points):
        self.color = color
        self.if_closed = if_closed
        self.border_width = border_width
        self.points = points

    def blit(self, screen):
        pygame.draw.lines(screen, self.color, self.if_closed, self.points, self.border_width)


class AALine:
    def __init__(self, color, if_closed, border_width, points):
        self.color = color
        self.if_closed = if_closed
        self.border_width = border_width
        self.points = points

    def blit(self, screen):
        pygame.draw.aaline(screen, self.color, self.if_closed, self.points, self.border_width)


class AALines:
    def __init__(self, color, if_closed, border_width, points):
        self.color = color
        self.if_closed = if_closed
        self.border_width = border_width
        self.points = points

    def blit(self, screen):
        pygame.draw.aalines(screen, self.color, self.if_closed, self.points, self.border_width)



import ctypes
from typing import Tuple, Callable, Any
import pygame
import threading
import clr
from System.Windows.Forms import *  # pycharm报语法错误
from System.Threading import Thread, ApartmentState, ThreadStart  # pycharm报语法错误

clr.AddReference('System.Windows.Forms')
clr.AddReference('System.Threading')

_app = Application  # pycharm报语法错误
_user32 = ctypes.windll.user32


class _Form(object):
    pass


class WebView(object):
    def __init__(self, window: pygame.Surface, size: Tuple[int, int] = (1200, 550), url: str = '',
                 script_errors_suppressed: bool = True, menu_enabled: bool = False):

        self._width, self._height = size

        form = _Form()
        threading.Thread(target=self._get_web, args=(form, self._width, self._height)).run()

        while True:
            try:
                ie = form.web
                break
            except AttributeError:
                pass

        ie.ScriptErrorsSuppressed = script_errors_suppressed
        self.ie_hwnd = int(str(ie.Handle))
        self.x, self.y = 0, 0
        _user32.SetParent(self.ie_hwnd, pygame.display.get_wm_info()['window'])  # 嵌入窗口
        _user32.MoveWindow(self.ie_hwnd, self.x, self.y, self._width, self._height, True)  # 移动窗口

        if url != '':
            ie.Navigate(url)
        self.ie = ie
        self.ie.IsWebBrowserContextMenuEnabled = menu_enabled
        self.ie.NewWindow += self._before_window

    @staticmethod
    def _get_web(form: _Form, width: int, height: int):
        web = WebBrowser()  # pycharm报语法错误
        form.web = web
        web.Width = width
        web.Height = height

    @staticmethod
    def _before_window(sender, e):
        href = sender.Document.ActiveElement.GetAttribute('href')
        sender.Navigate(href)
        e.Cancel = True

    @property
    def pos(self):
        return self.x, self.y

    @pos.setter
    def pos(self, set_pos: Tuple[int, int]):
        self.x, self.y = set_pos
        _user32.MoveWindow(self.ie_hwnd, self.x, self.y, self._width, self._height, True)

    def set_url(self, url: str):
        self.ie.Navigate(url)

    def show_url(self, func: Callable[[], Any]):
        self.ie.Navigating += func

    def resize(self, width: int, height: int):
        self._width, self._height = width, height
        self.ie.Width = width
        self.ie.Height = height

    def destroy(self):
        self.ie.Dispose()
        del self.ie

    def go_back(self):
        self.ie.GoBack()

    def go_forward(self):
        self.ie.GoForward()

    def refresh(self):
        self.ie.Refresh()



from SqrtS.Widgets.Widget import Widget
import pygame


class Image(Widget):
    """
    图片控件
    """

    def __init__(self, size,
                 IDname,
                 image_path):
        """

        :param size: [int,int]
        :param IDname: str
        :param image_path: str
        """
        super().__init__(size, IDname)
        self.image_path = image_path
        self.image_sur = pygame.transform.smoothscale(pygame.image.load(image_path), self.size)
        self.pos = []

    def bind_pos(self, new_pos: list):
        """
        绑定坐标
        :param new_pos: [int,int]
        :return: None
        """
        self.pos = new_pos

    def blit(self, window):
        """
        画图
        :param window:
        :return: None
        """
        window.get_window().blit(self.image_sur, self.pos)

    def register_for_event(self, window):
        """
        None
        :param window: None
        :return: None
        """
        ...

    def reload_image(self, path):
        """
        重新导入图片
        :param path: str
        :return: None
        """
        self.image_sur = pygame.image.load(path)

    def resize_image(self, new_size):
        """
        重新更改图片大小
        :param new_size:
        :return:
        """
        self.image_sur = pygame.transform.scale(self.image_sur, new_size)


class SVGImage(Widget):
    def __init__(self, size, IDname, svg_path):
        super().__init__(size, IDname)
        self.image_path = svg_path
        self.image_sur = pygame.image.load(svg_path)
        self.pos = []

    def bind_pos(self, new_pos: list):
        self.pos = new_pos

    def blit(self, window):
        """
        画图
        :param window:
        :return: None
        """
        window.get_window().blit(self.image_sur, self.pos)

    def register_for_event(self, window):
        ...


if __name__ == "__main__":
    svg = SVGImage([100, 100], "C:\\Users\\aa\\Desktop\\svgs\\")



import pygame


def draw_rect(screen, color, rect, border_width):
    pygame.draw.rect(screen, color, rect, border_width)


def draw_polygon(screen, color, pointlist, border_width):
   pygame.draw.polygon(screen, color, pointlist, width=border_width)





import pygame
from SqrtS.tools.load_theme import Theme
from SqrtS.Core.event_process import MOUSEUP, MOUSEDOWN, EVERYTIME
from SqrtS.Widgets import Widget
from SqrtS.Core.event_process import MOUSEUP, MOUSEDOWN, EVERYTIME

BROADWISE = 0
DIRECTION = 1


class Button(Widget.Widget):
    def __init__(self, button_size: list,
                 button_text: str,
                 theme: Theme,
                 font,
                 IDname: str,
                 style=None,
                 ):
        """

        :param button_size: [int,int]
        :param button_text: str
        :param theme: Theme
        :param font: SqrtSFont
        :param IDname: str
        :param style: [(int,int,int),(int,int,int)]
        """
        super().__init__(size=button_size, many_weights=False, IDname=IDname)
        if style is None:
            self.style = [(0, 0, 255), True, None]
        else:
            self.style = style

        self.event_types = [MOUSEDOWN, MOUSEUP, EVERYTIME]

        self.IDname = IDname

        self.button_text = button_text
        self.button_size = button_size

        # 获取按钮图像
        if theme is not None:
            self.button_formal = theme.get_theme()["button"]
            self.button_click = theme.get_theme()["button_click"]
            self.button_chosen = theme.get_theme()["button_chosen"]

        # 重新更改按钮大小
        self.button_formal = pygame.transform.scale(self.button_formal, button_size)
        self.button_click = pygame.transform.scale(self.button_click, button_size)
        self.button_chosen = pygame.transform.scale(self.button_chosen, button_size)
        self.button_img = self.button_formal
        # 设置按钮的坐标属性
        self.pos = []
        # 定义字体
        self.font = font
        # 按钮文本surface
        self.text_sur = None
        self.text_pos = [0, 0]
        self.text_sur = self.font.render_text(self.button_text, self.style[0], self.style[1], self.style[2])
        self.text_sur_size = self.text_sur.get_rect()

        self.call_function = None
        self.up_function = None
        self.move_on_function = None
        self.move_out_function = None

        self.if_this_time_call_on = False
        self.click = False
        self.chosen = False

    def check_mouse_down(self, event, window):
        """
        检测鼠标按下并且调用回调函数的函数（笑）
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.button_size[0] and self.pos[1] < mouse_click_pos[1] < \
                self.pos[1] + self.button_size[1]:
            self.button_img = self.button_click
            self.click = True
            if self.call_function:
                self.call_function(self)
        else:
            self.click = False

    def check_mouse_up(self, event, window):
        """
        检测鼠标抬起并且调用回调函数的函数（笑）
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.button_size[0] and self.pos[1] < mouse_click_pos[1] < \
                self.pos[1] + self.button_size[1]:
            self.button_img = self.button_formal
            self.click = False
            if self.up_function:
                self.up_function(self)
        else:
            self.click = False

    def weight_everytime_event(self, window):
        """
        每时每刻都在运行的函数为everytime函数，按钮内用于处理按钮移动上去的事件
        :return: None
        """
        mouse_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_pos[0] < self.pos[0] + self.button_size[0] and self.pos[1] < mouse_pos[1] < \
                self.pos[1] + self.button_size[1] and not self.click:
            self.button_img = self.button_chosen
            self.chosen = True
            if self.move_on_function and not self.if_this_time_call_on:
                self.move_on_function(self)
                self.if_this_time_call_on = True
        elif self.chosen:
            if self.move_out_function:
                self.move_out_function(self)
            self.chosen = False
            self.if_this_time_call_on = False
        else:
            self.button_img = self.button_formal

    def bind_pos(self, new_pos):
        """
        绑定新的按钮坐标
        :param new_pos:list
        :return: None
        """
        self.pos = new_pos
        if self.text_sur is not None:
            self._compute_text_pos(self.text_sur.get_rect())

    def register_for_event(self, window):
        """
        注册函数，将目标函数注册到EventProcessor中
        :param window: window对象
        :return: None
        """
        window.event_processor.register_event(MOUSEDOWN, self.check_mouse_down)
        window.event_processor.register_event(MOUSEUP, self.check_mouse_up)
        window.event_processor.register_event(EVERYTIME, self.weight_everytime_event)

    def _compute_text_pos(self, text_rect):
        """
        计算新的文字坐标
        :param text_rect:文字对象surface
        :return: 坐标列表
        """
        x = int((self.button_size[0] / 2) - (text_rect[2] / 2))
        y = int((self.button_size[1] / 2) - (text_rect[3] / 2))
        self.text_pos = [self.pos[0] + x, self.pos[1] + y]

    def blit_text(self, window):
        """
        画出控件的文本
        :return:None
        """
        window.get_window().blit(self.text_sur, self.text_pos)

    def blit(self, window):
        """
        画出控件
        :param window:窗口类
        :return: None
        """
        window.get_window().blit(self.button_img, self.pos)
        self.blit_text(window)

    def reset_text(self, new_text, new_style):
        """
        重设按钮文字大小
        :param new_text:更新的文字
        :param new_style: 更新的样式[颜色元组，是否抗锯齿，背景颜色（没有可设为None）]
        :return: None
        """
        self.button_text = new_text
        self.style = new_style
        self.text_sur = self.font.render_text(self.button_text, self.style[0], self.style[1], self.style[2])
        self.text_sur_size = self.text_sur.get_rect()

    def bind_call_func(self, func):
        """
        绑定回调函数
        :param func:函数体
        :return: None
        """
        self.call_function = func

    def bind_up_func(self, func):
        """
        绑定结束函数
        :param func:函数体
        :return: None
        """
        self.up_function = func

    def bind_on_func(self, func):
        """
        绑定鼠标一上去的函数
        :param func:函数体
        :return: None
        """
        self.move_on_function = func

    def bind_out_func(self, func):
        """
        绑定鼠标一出去的函数
        :param func:函数体
        :return: None
        """
        self.move_out_function = func


class ImageButton(Button):
    """
    图片按钮，继承自Button
    """

    def __init__(self, button_size: list,
                 image_list: list,
                 IDname: str):
        """
        图片按钮，用于创建没有文字，只有图片的按钮
        :param button_size: 按钮的大小
        :param image_list: ImageSurface对象的列表，第一个元素为正常时候的图片，第二个元素为按下时的图片，第三个元素为选中但不按下是的图片
        """
        self.widget_alive = True
        self.many_weight = False
        self.size = button_size
        self.button_size = button_size
        self.text_sur = None
        self.text_pos = None
        self.text_sur_size = None
        self.button_text = None
        self.IDname = IDname
        self.event_types = [MOUSEDOWN, MOUSEUP, EVERYTIME]

        # 获取按钮图像
        self.button_formal = image_list[0]
        self.button_click = image_list[1]
        self.button_chosen = image_list[2]
        # 重新更改按钮大小
        if not type(self.button_formal) == pygame.Surface:
            self.button_formal = pygame.transform.scale(self.button_formal.return_surface(), button_size)
            self.button_click = pygame.transform.scale(self.button_click.return_surface(), button_size)
            self.button_chosen = pygame.transform.scale(self.button_chosen.return_surface(), button_size)
            self.button_img = self.button_formal
        else:
            self.button_formal = pygame.transform.scale(self.button_formal, button_size)
            self.button_click = pygame.transform.scale(self.button_click, button_size)
            self.button_chosen = pygame.transform.scale(self.button_chosen, button_size)
            self.button_img = self.button_formal
        # 设置按钮的坐标属性
        self.pos = []

        self.call_function = None
        self.up_function = None
        self.move_on_function = None
        self.move_out_function = None

        self.if_this_time_call_on = False
        self.click = False
        self.chosen = False

    def _compute_text_pos(self, text_rect):
        ...

    def blit_text(self, window):
        ...

    def reset_text(self, new_text, new_style):
        ...


class ImageTextButton(ImageButton):
    def __init__(self, button_size: list,
                 image_list: list,
                 IDname: str,
                 button_text,
                 font,
                 style=None):
        """

        :param button_size: [int,int]
        :param image_list: [IS,IS,IS]
        :param IDname: str
        :param button_text:str
        :param font: SqrtSSFont
        :param style: [(),()]
        """
        super().__init__(button_size, image_list, IDname)
        if style is None:
            self.style = [(0, 0, 255), True, None]
        else:
            self.style = style
        self.button_text = button_text
        self.event_types = [MOUSEDOWN, MOUSEUP, EVERYTIME]
        self.font = font
        # 按钮文本surface
        self.text_sur = None
        self.text_pos = [0, 0]
        self.text_sur = self.font.render_text(self.button_text, self.style[0], self.style[1], self.style[2])
        self.text_sur_size = self.text_sur.get_rect()

    def _compute_text_pos(self, text_rect):
        """
        计算新的文字坐标
        :param text_rect:文字对象surface
        :return: 坐标列表
        """
        x = int((self.button_size[0] / 2) - (text_rect[2] / 2))
        y = int((self.button_size[1] / 2) - (text_rect[3] / 2))
        self.text_pos = [self.pos[0] + x, self.pos[1] + y]

    def blit_text(self, window):
        """
        画出控件的文本
        :return:None
        """
        window.get_window().blit(self.text_sur, self.text_pos)

    def reset_text(self, new_text, new_style):
        """
        重设按钮文字大小
        :param new_text:更新的文字
        :param new_style: 更新的样式[颜色元组，是否抗锯齿，背景颜色（没有可设为None）]
        :return: None
        """
        self.button_text = new_text
        self.style = new_style
        self.text_sur = self.font.render_text(self.button_text, self.style[0], self.style[1], self.style[2])
        self.text_sur_size = self.text_sur.get_rect()


class CheckBox(ImageButton):
    def __init__(self,
                 button_size: list,
                 theme: Theme,
                 IDname: str,
                 image_list=None
                 ):
        """

        :param button_size: [int,int]
        :param theme: Theme
        :param IDname: str
        :param image_list:[IS,IS,IS]
        """

        if not image_list:
            image_list = [
                theme.get_theme()["checkbox"],
                theme.get_theme()["checkbox_click"],
                theme.get_theme()["checkbox_chosen"]
            ]
        else:
            image_list = image_list

        super().__init__(button_size=button_size, image_list=image_list, IDname=IDname)

        self.if_chose = False

    def check_mouse_down(self, event, window):
        """
        检测鼠标按下并且调用回调函数的函数（笑）
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.button_size[0] and self.pos[1] < mouse_click_pos[1] < \
                self.pos[1] + self.button_size[1]:
            self.click = True
            self.if_chose = not self.if_chose
            if self.if_chose:
                self.button_img = self.button_click
            else:
                self.button_img = self.button_chosen
            if self.call_function:
                self.call_function(self)
        else:
            self.click = False

    def check_mouse_up(self, event, window):
        """
        检测鼠标抬起并且调用回调函数的函数（笑）
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.button_size[0] and self.pos[1] < mouse_click_pos[1] < \
                self.pos[1] + self.button_size[1]:
            self.click = False
            if self.up_function:
                self.up_function(self)
        else:
            self.click = False

    def weight_everytime_event(self, window):
        """
        每时每刻都在运行的函数为everytime函数，按钮内用于处理按钮移动上去的事件
        :return: None
        """
        mouse_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_pos[0] < self.pos[0] + self.button_size[0] and self.pos[1] < mouse_pos[1] < \
                self.pos[1] + self.button_size[1] and not self.click and not self.if_chose:
            self.button_img = self.button_chosen
            self.chosen = True
            if self.move_on_function and not self.if_this_time_call_on:
                self.move_on_function()
                self.if_this_time_call_on = True
        elif self.click:
            ...
        elif self.chosen:
            if self.move_out_function:
                self.move_out_function()
            self.chosen = False
            self.if_this_time_call_on = False
        elif self.if_chose:
            ...
        else:
            self.button_img = self.button_formal
            self.click = False


class _RadioButton(ImageButton):
    def __init__(self,
                 button_size: list,
                 theme: Theme,
                 group,
                 IDname: str,
                 index: int):
        """

        :param button_size: [int,int]
        :param theme: Theme
        :param group: SqrtSRadioButtonGroup
        :param IDname: str
        :param index: int
        """
        self.radiobutton_group = group
        self.index = index
        image_list = [
            theme.get_theme()["radiobutton"],
            theme.get_theme()["radiobutton_click"],
            theme.get_theme()["radiobutton_chosen"]
        ]

        super().__init__(button_size=button_size, image_list=image_list, IDname=IDname)
        self.if_chose_radio = False

    def check_mouse_down(self, event, window):
        """
        检测鼠标按下并且调用回调函数的函数（笑）
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.button_size[0] and self.pos[1] < mouse_click_pos[1] < \
                self.pos[1] + self.button_size[1]:
            self.click = True
            self.if_chose_radio = True
            for i in self.radiobutton_group.radio_buttons:
                if i.index != self.index and i.if_chose_radio:
                    i.if_chose_radio = False
                    i.button_img = i.button_formal
                    self.radiobutton_group.final_chose = self.index
            if self.if_chose_radio:
                self.button_img = self.button_click
            else:
                self.button_img = self.button_chosen
            if self.call_function:
                self.call_function(self)
        else:
            self.click = False

    def check_mouse_up(self, event, window):
        """
        检测鼠标抬起并且调用回调函数的函数（笑）
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.button_size[0] and self.pos[1] < mouse_click_pos[1] < \
                self.pos[1] + self.button_size[1]:
            self.click = False
            if self.up_function:
                self.up_function()
        else:
            self.click = False

    def weight_everytime_event(self, window):
        """
        每时每刻都在运行的函数为everytime函数，按钮内用于处理按钮移动上去的事件
        :return: None
        """
        mouse_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_pos[0] < self.pos[0] + self.button_size[0] and self.pos[1] < mouse_pos[1] < \
                self.pos[1] + self.button_size[1] and not self.click and not self.if_chose_radio:
            self.button_img = self.button_chosen
            self.chosen = True
            if self.index != self.radiobutton_group.final_chose and self.if_chose_radio:
                self.if_chose_radio = False
                self.button_img = self.button_formal
            if self.move_on_function and not self.if_this_time_call_on:
                self.move_on_function()
                self.if_this_time_call_on = True
        elif self.click:
            ...
        elif self.chosen:
            if self.move_out_function:
                self.move_out_function()
            self.chosen = False
            self.if_this_time_call_on = False
        elif self.if_chose_radio:
            ...
        else:
            self.button_img = self.button_formal


class SqrtSRadioButtonGroup(Widget.Widget):
    def __init__(self,
                 button_numbers,
                 button_size: list,
                 theme: Theme,
                 IDname: str,
                 default=0,
                 padding=5):
        self.button_size = button_size
        self.button_numbers = button_numbers
        self.radio_buttons = [_RadioButton(button_size, theme, self, IDname + str(i), i) for i in range(button_numbers)]
        self.final_chose = default
        self.radio_buttons[default].if_chose_radio = True
        self.pos = []
        self.padding = padding
        self.call_function = None
        self.up_function = None
        self.move_on_function = None
        self.move_out_function = None

        super().__init__(button_size, many_weights=True, IDname=IDname)

    def blit(self, window):
        for i in self.radio_buttons:
            i.blit(window)

    def register_for_event(self, window):
        for i in self.radio_buttons:
            i.register_for_event(window)

    def bind_pos(self, new_pos: list):
        """
        绑定新的按钮坐标
        :param new_pos:list
        :return: None
        """
        self.pos = new_pos
        self.update_radio_buttons_pos()

    def bind_call_func(self, func):
        """
        绑定回调函数
        :param func:函数体
        :return: None
        """
        self.call_function = func
        for i in self.radio_buttons:
            i.bind_call_func(self.call_function)

    def bind_up_func(self, func):
        """
        绑定结束函数
        :param func:函数体
        :return: None
        """
        self.up_function = func
        for i in self.radio_buttons:
            i.bind_call_func(self.up_function)

    def bind_on_func(self, func):
        """
        绑定鼠标一上去的函数
        :param func:函数体
        :return: None
        """
        self.move_on_function = func
        for i in self.radio_buttons:
            i.bind_call_func(self.move_on_function)

    def bind_out_func(self, func):
        """
        绑定鼠标一出去的函数
        :param func:函数体
        :return: None
        """
        self.move_out_function = func
        for i in self.radio_buttons:
            i.bind_call_func(self.move_out_function)

    def update_radio_buttons_pos(self):
        for index, i in enumerate(self.radio_buttons):
            i_pos = [
                self.pos[0],
                self.pos[1] + (self.button_size[1] + self.padding) * index,
            ]
            i.bind_pos(i_pos)
        self.pos = self.radio_buttons[-1].pos
        self.size = [self.button_size[0], (self.button_size[1] + self.padding) * self.button_numbers]

    def process_many_weight_poses(self):
        self.size = [self.size[0], (self.size[1] + self.padding) * self.button_numbers]


class SwitchButton1(CheckBox):
    def __init__(self, button_size: list,
                 theme: Theme,
                 IDname: str):
        """

        :param button_size: [int,int]
        :param theme: Theme
        :param IDname: str
        """
        image_list = [
            theme.get_theme()["switch"],
            theme.get_theme()["switch_click"],
            theme.get_theme()["switch"]
        ]
        self.IDname = IDname
        super().__init__(button_size, theme, image_list=image_list, IDname=IDname)


class SwitchButton2(CheckBox):
    def __init__(self, button_size: list,
                 theme: Theme,
                 IDname: str):
        """

        :param button_size: [int,int]
        :param theme: Theme
        :param IDname: str
        """
        image_list = [
            theme.get_theme()["switch2"],
            theme.get_theme()["switch2_click"],
            theme.get_theme()["switch2"]
        ]
        self.IDname = IDname
        super().__init__(button_size, theme, image_list=image_list, IDname=IDname)


# ——————————————————————不确定——————————————————————
class _SlideButton(ImageButton):
    def __init__(self, button_size,
                 img_list,
                 limit_direction,
                 IDname: str):
        super().__init__(button_size, img_list, IDname)
        self.pos = []
        self.motion_range = []
        self.move_length = 0
        self.limit_forward = limit_direction
        if limit_direction == 0:
            self.other_direction = 1
        else:
            self.other_direction = 0

    def blit(self, window):
        window.get_window().blit(self.button_img, self.pos)

    def bind_pos(self, new_pos):
        self.pos = new_pos

    def bind_move_length(self, pos, length):
        motion_range = [pos[0], pos[1] + length]
        self.move_length = length
        self.motion_range = motion_range

    def check_mouse_down(self, event, window):
        """
        检测鼠标按下并且调用回调函数的函数（笑）
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.button_size[0] and self.pos[1] < mouse_click_pos[1] < \
                self.pos[1] + self.button_size[1]:
            self.button_img = self.button_click
            self.click = True
            if self.call_function:
                self.call_function(self)

    def check_mouse_up(self, event, window):
        """
        检测鼠标抬起并且调用回调函数的函数（笑）
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.button_size[0] and self.pos[1] < mouse_click_pos[1] < \
                self.pos[1] + self.button_size[1]:
            self.button_img = self.button_formal
            self.click = False
            if self.up_function:
                self.up_function(self)

    def weight_everytime_event(self, window):
        """
        每时每刻都在运行的函数为everytime函数，按钮内用于处理按钮移动上去的事件
        :return: None
        """
        self.check_pos()
        mouse_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_pos[0] < self.pos[0] + self.button_size[0] and self.pos[1] < mouse_pos[1] < \
                self.pos[1] + self.button_size[1] and not self.click:
            self.button_img = self.button_chosen
            self.chosen = True
            if self.move_on_function and not self.if_this_time_call_on:
                self.move_on_function(self)
                self.if_this_time_call_on = True
        elif self.click:
            if not self.pos[self.limit_forward] < mouse_pos[self.limit_forward] < self.pos[self.limit_forward] + \
                   self.size[self.limit_forward]:
                self.click = False
            if not self.pos[self.other_direction] < mouse_pos[self.other_direction] < self.pos[self.other_direction] + \
                   self.size[self.other_direction]:
                self.click = False
            if self.motion_range[0] < mouse_pos[self.limit_forward] < self.motion_range[1]:
                self.pos[self.limit_forward] = mouse_pos[self.limit_forward] - int(self.size[self.limit_forward] / 2)

        elif self.chosen:
            if self.move_out_function:
                self.move_out_function(self)
            self.chosen = False
            self.if_this_time_call_on = False
        else:
            self.button_img = self.button_formal

    def check_pos(self):
        if self.pos[0] < self.motion_range[0]:
            self.pos[0] = self.motion_range[0]
        if self.pos[0] > self.motion_range[1] - self.button_size[0]:
            self.pos[0] = self.motion_range[1] - self.button_size[0]

    def get_pos(self):
        """获取坐标"""
        return self.pos

    def get_move_length(self):
        """
        获得移动长度
        :return:
        """
        return self.move_length



import threading

"""
我想你来到这又是一脸懵对不对？怎么前面一个事件处理器，这么这里又来了一个任务处理器？
这个不同于事件处理器，这个是处理用户定义的任务的。
比如，你有一个任务想要长期执行，每间隔多少秒，那么你就可以调用register_for_infinite_task啦！是不是很方便？
亦或者，你想执行一个任务，间隔多少秒，你就可以调用register_for_times_task
注意！这里的注册函数，一旦注册便会开始计数！也就是一旦注册便会进行运行倒计时！

from:__CEMEYE__  
:)
"""

ONCE = 111
TIMES = 222
INFINITE = 333
TERMINABLE_INFINITE = 444
PAUSING = 555


class TaskSystem:
    """
    挺无聊的类，我也不想去额外定义一个任务类了
    """

    def __init__(self):
        self.infinite_task_pool = []
        self.once_task_pool = []
        self.times_task_pool = []
        self.pausing_task_pool = []
        self.pausing_task_names = {}

        self.terminable_infinite_task = []
        self.terminable_task_names = {}

    def register_for_infinite_task(self, task_func, time_interval):
        """
        注册永久任务
        :param task_func: 目标函数
        :param time_interval: 时间间隔(s)
        :return: None
        """
        self.infinite_task_pool.append([task_func, time_interval])

    def register_for_terminable_infinite_task(self, task_func, time_interval, task_name):
        """
        注册永久任务
        :param task_name: 可停止无限函数的ID名
        :param task_func: 目标函数
        :param time_interval: 时间间隔(s)
        :return: None
        """
        self.terminable_infinite_task.append([task_func, time_interval, task_name])
        self.terminable_task_names[task_name] = True

    def register_fot_once_task(self, task_func, time_interval, args=None):
        """
        注册一次性任务
        :param task_func:任务函数
        :param time_interval: 任务间隔（多少秒后执行？）
        :param args: 参数，可选
        :return: None
        """
        self.once_task_pool.append([task_func, time_interval, args])

    def register_for_times_task(self, task_func, times, time_interval, args):
        """
        注册多次的任务
        :param args: 参数
        :param task_func: 任务函数体
        :param times: 次数
        :param time_interval:时间间隔
        :return:None
        """
        self.times_task_pool.append([task_func, time_interval, times, args])

    def register_for_pausing_task(self, task_func, time_interval, task_name):
        """
        注册暂停时期的任务，时间间隔可以为0
        :param task_func: 任务函数体
        :param time_interval:时间间隔
        :return:None
        """
        self.pausing_task_pool.append([task_func, time_interval, task_name])
        self.pausing_task_names[task_name] = True

    def infinite_call(self, time_interval, func):
        """
        内部方法，无线调用函数
        :param time_interval:时间间隔
        :param func: 函数体
        :return: None
        """
        func()
        threading.Timer(time_interval, self.infinite_call, args=[time_interval, func]).start()

    def terminable_infinite_call(self, time_interval, func, task_name):
        func()
        if self.terminable_task_names[task_name]:
            threading.Timer(time_interval, self.terminable_infinite_call, args=[time_interval, func, task_name]).start()

    def terminable_infinite_pausing_call(self, time_interval, func, task_name):
        func()
        if self.pausing_task_names[task_name]:
            threading.Timer(time_interval, self.terminable_infinite_pausing_call,
                            args=[time_interval, func, task_name]).start()

    def times_call(self, time_interval, times_, func, i, args):
        """
        内部函数，用于多次调用，内部采用递归实现
        :param time_interval: 时间间隔
        :param times_: 次数
        :param func: 函数体
        :param i: 这个函数被for循环调用，所以需要传i
        :return: None
        """
        times_ = times_
        count = 0

        def _times_call():
            """
            内部函数
            :return:None
            """
            nonlocal count
            count += 1
            func()
            if count < times_:
                t = threading.Timer(time_interval, _times_call, args=[args])
                t.start()
            else:
                try:
                    self.times_task_pool.remove(i)
                except ValueError as e:
                    ...

        _times_call()

    def task_system_update(self):
        """
        任务系统更新，内部函数
        :return: None
        """
        for i in self.once_task_pool:
            threading.Timer(i[1], i[0], args=[i[2], ]).start()
            self.once_task_pool.remove(i)

        for i in self.times_task_pool:
            self.times_call(i[1], i[2], i[0], i, args=i[-1])
            self.times_task_pool.remove(i)

        for i in self.infinite_task_pool:
            self.infinite_call(i[1], i[0])
            self.infinite_task_pool.remove(i)

        for i in self.terminable_infinite_task:
            self.terminable_infinite_call(i[1], i[0], i[2])
            self.terminable_infinite_task.remove(i)

    def start_pausing_tasks(self):
        """
        暂停时期的任务系统更新，内部函数
        :return: None
        """
        for i in self.pausing_task_pool:
            self.terminable_infinite_pausing_call(i[1], i[0], i[2])
            self.pausing_task_pool.remove(i)

    def destroy_pausing_type(self):
        """
        停止暂停状态
        :return: None
        """
        for key in self.pausing_task_names.keys():
            self.pausing_task_names[key] = False

    def destroy_terminable_infinite_pausing_task(self, task_name):
        """
        注销某个暂停时期的可停止的任务
        :param task_name:任务名
        :return: None
        """
        self.pausing_task_names[task_name] = False

    def destroy_terminable_infinite_task(self, task_name):
        """
        注销某个可停止的任务
        :param task_name:任务名
        :return: None
        """
        self.terminable_task_names[task_name] = False


if __name__ == "__main__":
    def once():
        print("once task!")


    def times():
        print("times!")


    def infinite():
        print("infinite!")


    def async_register(ts):
        ts.register_for_times_task(times, 2, 3)


    ts = TaskSystem()

    threading.Timer(0.01, async_register, args=[ts]).start()

    ts.register_for_infinite_task(infinite, 0.01)

    while True:
        ts.task_system_update()



import time
import threading


class Timer:
    """
    定时器类
    """

    def __init__(self, put: int, call_func):
        self.alive = True
        self.put = put
        self.start_time = int(time.time())
        self.passtime = 0
        self.call_func = call_func

    def check_time(self):
        while self.alive:
            self.passtime = int(time.time()) - self.start_time
            if self.put <= self.passtime:
                self.call_func()
                self.alive = False

    def run_timer(self):
        thread = threading.Thread(target=self.check_time)
        thread.start()

    def stop(self):
        self.alive = False


class AccurateTimer:
    def __init__(self, put: int, call_func):
        self.alive = True
        self.put = put
        self.start_time = int(time.time() * 1000)
        self.passtime = 0
        self.call_func = call_func

    def check_time(self):
        while self.alive:
            self.passtime = int(time.time() * 1000) - self.start_time
            if self.put <= self.passtime:
                self.call_func()
                self.alive = False

    def run_timer(self):
        thread = threading.Thread(target=self.check_time)
        thread.start()

    def stop(self):
        self.alive = False


class CirculateTimer:
    def __init__(self, put: int, call_func):
        self.alive = True
        self.put = put
        self.start_time = int(time.time())
        self.passtime = 0
        self.call_func = call_func

    def check_time(self):
        while self.alive:
            self.passtime = int(time.time()) - self.start_time
            if self.put <= self.passtime:
                self.call_func()
                self.start_time = int(time.time())

    def run_timer(self):
        thread = threading.Thread(target=self.check_time)
        thread.start()

    def stop(self):
        self.alive = False


class AccurateCirculateTimer:
    def __init__(self, put: int, call_func):
        self.alive = True
        self.put = put
        self.start_time = int(time.time() * 1000)
        self.passtime = 0
        self.call_func = call_func

    def check_time(self):
        while self.alive:
            self.passtime = int(time.time() * 1000) - self.start_time
            if self.put <= self.passtime:
                self.call_func()
                self.start_time = int(time.time() * 1000)

    def run_timer(self):
        thread = threading.Thread(target=self.check_time)
        thread.start()

    def stop(self):
        self.alive = False


class TimesCirculateTimer:
    def __init__(self, put: int, call_func, times):
        self.times = times
        self.alive = True
        self.put = put
        self.start_time = int(time.time() * 1000)
        self.passtime = 0
        self.call_func = call_func

    def check_time(self):
        while self.alive and self.times:
            self.passtime = int(time.time() * 1000) - self.start_time
            if self.put <= self.passtime:
                self.call_func()
                self.start_time = int(time.time() * 1000)
                self.times -= 1

    def run_timer(self):
        thread = threading.Thread(target=self.check_time)
        thread.start()

    def stop(self):
        self.alive = False


if __name__ == "__main__":
    def say_hello():
        print("hello!")


    t = TimesCirculateTimer(500, say_hello, 5)
    t.run_timer()



from SqrtS.Layouts.Layout import Layout
from SqrtS.Widgets.Widget import Widget


class LinearLayout(Layout):
    """
    线性布局，全部一个一个罗列下来的布局
    """

    def __init__(self, window_size, name, weight_spacing=3, direction="lengthwise"):
        super().__init__(name=name)
        self.window_size = window_size
        self.weights = []

        self.weight_spacing = weight_spacing

        self.y_limit = 0

        self.direction = direction

    def append(self, weight: Widget):
        """
        添加新的控件
        :param weight:控件
        :return:None
        """
        self.weights.append(weight)

    def compute_poses(self):
        """
        线性布局的坐标更新计算方式
        :return: None
        """
        # 纵向布局的坐标更新方法
        if self.direction == "lengthwise":
            for index, i in enumerate(self.weights):
                if i.many_weight:
                    i.process_many_weight_poses()
                    # i.size = [i.size[0], (i.size[1] + i.padding) * i.button_numbers]
                size_i = i.size
                half_length = int(size_i[0] / 2)
                window_x_center = int(self.window_size[0] / 2)
                i_pos_x = window_x_center - half_length
                i_pos_y = self.y_limit
                self.y_limit += size_i[1]
                i_pos = [i_pos_x, i_pos_y + int(self.weight_spacing) * index]
                i.bind_pos(i_pos)

        # 横向布局的坐标更新方法
        elif self.direction == "broadwise":
            for index, i in enumerate(self.weights):
                if i.many_weight:
                    i.process_many_weight_poses()
                    # i.size = [i.size[0], (i.size[1] + i.padding) * i.button_numbers]
                size_i = i.size
                half_width = int(size_i[1] / 2)
                window_y_center = int(self.window_size[1] / 2)
                i_pos_x = self.y_limit
                i_pos_y = window_y_center - half_width
                self.y_limit += size_i[0]
                i_pos = [i_pos_x + int(self.weight_spacing) * index, i_pos_y]
                i.bind_pos(i_pos)

    def blit_all(self, window):
        """
        在屏幕上画出所有的控件，通过隔代调用控件的绘图方法
        |这个方法会在Window类中被调用|
        :param window: Window类对象
        :return: None
        """
        for i in self.weights:
            i.blit(window)

    def appends(self, weights: list):
        """
        批量添加控件
        :param weights: 控件列表
        :return:
        """
        for i in weights:
            self.append(i)
        self.compute_poses()


if __name__ == "__main__":
    ...



from SqrtS.Widgets.Label import LineLabel, Label
from SqrtS.Widgets.FrameBox import FrameBox
from SqrtS.Widgets.Widget import Widget
from SqrtS.Core.TaskSystem import TIMES
import pygame


class Bubble(Widget):
    def __init__(self, size,
                 theme,
                 title_font,
                 message_font,
                 title,
                 message,
                 IDname,
                 title_color=(255, 255, 255),
                 message_color=(255, 255, 255),
                 title_background=None,
                 message_background=None):
        """

        :param size: []
        :param theme: Theme
        :param title_font: SqrtFont
        :param message_font: SqrtFont
        :param title: str
        :param message: str
        :param IDname: str
        :param title_color:str
        :param message_color:(int,int,int)
        :param title_background: (int,int,int)
        :param message_background: (int,int,int)
        """
        super().__init__(size, IDname)
        line_length = int(size[0] / message_font.font_size)
        self.alive = True
        self.pos = []
        self.size = size
        self.IDname = IDname
        self.bubble_img = pygame.transform.scale(theme.get_theme()["bubble"], size)
        self.frame_box = FrameBox("framebox>" + IDname)

        self.title_label = Label(title, title_color, title_font, "label>" + self.IDname, background=title_background)
        self.message_label = LineLabel(message, message_color, message_font, line_length, "label2>" + IDname,
                                       background=message_background)

        self.frame_box.appends_widgets([self.title_label, self.message_label])
        self.frame_box.bind_widgets_poses({"label>" + self.IDname: [10, 10], "label2>" + self.IDname: [10, 50]})

    def bind_pos(self, new_pos: list):
        self.pos = new_pos
        self.frame_box.bind_pos(self.pos)

    def blit(self, window):
        window.get_window().blit(self.bubble_img, self.pos)
        self.frame_box.blit(window)

    def register_for_event(self, window):
        self.frame_box.register_for_event(window)
        window.register_for_task(TIMES, self.update, args=window,times=10)

    def update(self, window):
        ...

    def raise_bubble(self, window):
        window_size = window.get_window_size()
        self.bind_pos([window_size[0] - self.size[0], window_size[1]])



import pygame
from SqrtS.Widgets.Widget import Widget
from SqrtS.Core.event_process import MOUSEUP, MOUSEDOWN, EVERYTIME, KEYDOWN, TEXTINPUT, MOUSE_BUTTON_LEFT_DOWN, \
    MOUSE_BUTTON_LEFT_UP
from SqrtS.Core.TaskSystem import INFINITE

# 按键映射字典表
KEYMAP = {
    pygame.K_a: "a", pygame.K_b: "b", pygame.K_c: "c", pygame.K_d: "d", pygame.K_e: "e", pygame.K_f: "f",
    pygame.K_g: "g", pygame.K_h: "h", pygame.K_i: "i", pygame.K_j: "j", pygame.K_k: "k", pygame.K_l: "l",
    pygame.K_m: "m", pygame.K_n: "n", pygame.K_o: "o", pygame.K_p: "p", pygame.K_q: "q", pygame.K_r: "r",
    pygame.K_s: "s", pygame.K_t: "t", pygame.K_u: "u", pygame.K_v: "v", pygame.K_w: "w", pygame.K_x: "x",
    pygame.K_y: "y", pygame.K_z: "z",

    pygame.K_KP0: "0", pygame.K_KP1: "1", pygame.K_KP2: "2", pygame.K_KP3: "3", pygame.K_KP4: "4",
    pygame.K_KP5: "5", pygame.K_KP6: "6", pygame.K_KP7: "7", pygame.K_KP8: "8", pygame.K_KP9: "9",

    pygame.K_0: "0", pygame.K_1: "1", pygame.K_2: "2", pygame.K_3: "3", pygame.K_4: "4", pygame.K_5: "5",
    pygame.K_6: "6", pygame.K_7: "7", pygame.K_8: "8", pygame.K_9: "9",

    pygame.K_TAB: "    ", pygame.K_COMMA: ",", pygame.K_PERIOD: ".", pygame.K_SLASH: "/", pygame.K_SEMICOLON: ";",
    pygame.K_BACKQUOTE: "`", pygame.K_QUOTE: "'", pygame.K_SPACE: " ",
    pygame.K_LEFTBRACKET: "[", pygame.K_RIGHTBRACKET: "]", pygame.K_BACKSLASH: "\\",

    pygame.K_PLUS: "+", pygame.K_MINUS: "-", pygame.K_EQUALS: "=",

    pygame.K_KP_MULTIPLY: "*", pygame.K_KP_PLUS: "+", pygame.K_KP_MINUS: "-", pygame.K_KP_EQUALS: "=",
    pygame.K_KP_DIVIDE: "/", pygame.K_KP_PERIOD: "."
}

# 热键映射字典表
HOTKEY = [
    pygame.K_LSHIFT,
    pygame.K_RSHIFT,
    pygame.K_LCTRL,
    pygame.K_RCTRL,
    pygame.K_LALT,
    pygame.K_RALT
]

# 大写转换对应表
UperCheck = {
    pygame.K_a: "A", pygame.K_b: "B", pygame.K_c: "C", pygame.K_d: "D", pygame.K_e: "E", pygame.K_f: "F",
    pygame.K_g: "G", pygame.K_h: "H", pygame.K_i: "I", pygame.K_j: "J", pygame.K_k: "K", pygame.K_l: "L",
    pygame.K_m: "M", pygame.K_n: "N", pygame.K_o: "O", pygame.K_p: "P", pygame.K_q: "Q", pygame.K_r: "R",
    pygame.K_s: "S", pygame.K_t: "T", pygame.K_u: "U", pygame.K_v: "V", pygame.K_w: "W", pygame.K_x: "X",
    pygame.K_y: "Y", pygame.K_z: "Z",

    pygame.K_0: ")", pygame.K_1: "!", pygame.K_2: "@", pygame.K_3: "#", pygame.K_4: "$", pygame.K_5: "%",
    pygame.K_6: "^", pygame.K_7: "&", pygame.K_8: "*", pygame.K_9: "(",

    pygame.K_COMMA: "<", pygame.K_PERIOD: ">", pygame.K_SLASH: "?", pygame.K_SEMICOLON: ":",
    pygame.K_BACKQUOTE: "~", pygame.K_QUOTE: "\"",
    pygame.K_LEFTBRACKET: "{", pygame.K_RIGHTBRACKET: "}", pygame.K_BACKSLASH: "|",

    pygame.K_MINUS: "_", pygame.K_EQUALS: "+",

}


def check_hotkey(keys):
    """
    检测是否是热键
    :param keys: []
    :return: bool
    """
    for i in HOTKEY:
        if keys[i]:
            return True
    return False


class TextInput(Widget):
    """
    输入框基类
    """

    def __init__(self,
                 size,
                 IDname,
                 theme,
                 font,
                 text_color=(255, 255, 255),
                 text_background=None,
                 cursor_color=(255, 0, 0)):
        """

        :param size: [int,int]
        :param IDname: str
        :param theme: Theme
        :param font: Font
        :param text_color:[int,int,int]
        :param text_background: [int,int,int]
        :param cursor_color: [int,int,int]
        """
        super().__init__(size, IDname)
        self.up_function = None
        self.call_function = None
        self.text_color = text_color
        self.text_background = text_background
        self.cursor_color = cursor_color
        self.size = size
        self.pos = [0, 0]
        self.font = font

        self.formal_input_image = pygame.transform.scale(theme.get_theme()["formal_input"], self.size)
        self.chosen_input_image = pygame.transform.scale(theme.get_theme()["chosen_input"], self.size)

        self.input_img = self.formal_input_image

        self.input_text_list = []

        self.alive = False

        self.hot_key_dict = {}

        self.cursor_pos = [0, 0]
        self.cursor_if_blit = False
        self.event_types = [MOUSEDOWN, MOUSEUP, EVERYTIME, KEYDOWN]

    def return_input_text(self):
        """
        返回现在有的文字
        :return: None
        """
        str_text = ""
        for i in self.input_text_list:
            str_text += i
        return str_text

    def clear_text(self):
        """
        清空文字
        :return:None
        """
        self.input_text_list = []
        self.cursor_pos = self.pos

    def _compute_cursor_pos(self, rect):
        """
        重新计算光标坐标
        :param rect:
        :return:
        """
        self.cursor_pos = [self.pos[0] + rect[2], self.pos[1]]

    def bind_pos(self, new_pos: list):
        """
        绑定坐标
        :param new_pos:[]
        :return:
        """
        self.pos = new_pos
        self.cursor_pos = new_pos

    def _blit_cursor(self, screen):
        """
        光标绘画
        :return:NONE
        """
        if self.cursor_if_blit and self.alive:
            start_pos = [self.cursor_pos[0], self.cursor_pos[1] + 5]
            end_pos = [self.cursor_pos[0], self.cursor_pos[1] + self.size[1] - 5]
            pygame.draw.line(screen, self.cursor_color, start_pos, end_pos)

    def _change_cursor(self):
        """
        被当做任务调用！
        :return: None
        """
        self.cursor_if_blit = not self.cursor_if_blit

    def render_text(self):
        """
        递归判断是否当前文字长度超出范围
        :return:
        """
        text = ""
        for i in self.input_text_list:
            text += i
        text_surface = self.font.render_text(text, self.text_color, background=self.text_background)
        self._compute_cursor_pos(text_surface.get_rect())
        return text_surface

    def blit(self, window):
        """
        绘图
        :param window:
        :return:
        """
        screen = window.get_window()
        screen.blit(self.input_img, self.pos)
        self._blit_cursor(screen)
        if len(self.input_text_list):
            screen.blit(self.render_text(), [self.pos[0], self.pos[1]])

    def check_mouse_down(self, event, window):
        """
        检测鼠标按下并且调用回调函数的函数（笑）
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.size[0] and self.pos[1] < mouse_click_pos[1] < \
                self.pos[1] + self.size[1]:
            self.input_img = self.chosen_input_image
            self.alive = True
            if self.call_function:
                self.call_function(self)
        else:
            self.alive = False

    def check_mouse_up(self, event, window):
        """
        检测鼠标抬起并且调用回调函数的函数（笑）
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.size[0] and self.pos[1] < mouse_click_pos[1] < \
                self.pos[1] + self.size[1]:
            self.input_img = self.chosen_input_image
            self.alive = True
            if self.up_function:
                self.up_function(self)
        else:
            self.input_img = self.formal_input_image
            self.alive = False

    def register_for_event(self, window):
        """
        注册函数，将目标函数注册到EventProcessor中
        :param window: window对象
        :return: None
        """
        window.event_processor.register_event(MOUSEDOWN, self.check_mouse_down)
        window.event_processor.register_event(MOUSEUP, self.check_mouse_up)
        window.event_processor.register_event(EVERYTIME, self.weight_everytime_event)
        window.event_processor.register_event(KEYDOWN, self.check_key_down)
        # window.event_processor.register_event(KEYUP, self.check_key_up)
        window.register_for_task(INFINITE, self._change_cursor, 0.2)

    def process_ctrl_hotkey(self, key):
        """
        处理ctrl的快捷键
        :param key:
        :return:
        """
        if key == pygame.K_c:
            print("这里执行复制的操作....")
        elif key == pygame.K_v:
            print("这里执行粘贴的操作.....")
        elif key == pygame.K_x:
            print("这里执行剪切的操作.......")
        elif key == pygame.K_a:
            print("这里执行全选的操作.........")
        elif key == pygame.K_b:
            print("这里执行什么也不知道的操作....(只是cemeye想要炫耀一下这个可以随意定义快捷键的功能啦~)")

        # 这个是用来执行自定义的快捷键的（仅限有关Ctrl的）
        for i in self.hot_key_dict.keys():
            if pygame.K_LCTRL in i or pygame.K_RCTRL in i:
                if key in i:
                    self.hot_key_dict[i]()

    def process_alt_hotkey(self, key):
        """
        处理alt的快捷键
        :param key:
        :return:
        """
        # 这个是用来执行自定义的快捷键的（仅限有关alt的）
        for i in self.hot_key_dict.keys():
            if pygame.K_LALT in i or pygame.K_RALT in i:
                if key in i:
                    self.hot_key_dict[i]()

    def process_shift_hotkey(self, key):
        """
        处理shift的快捷键
        :param key:
        :return:
        """
        # 这个是用来执行自定义的快捷键的（仅限有关shift的）
        for i in self.hot_key_dict.keys():
            if pygame.K_LSHIFT in i or pygame.K_RSHIFT in i:
                if key in i:
                    self.hot_key_dict[i]()

    def check_key_down(self, event, window):
        """
        检测是否有按键按下
        :param event: Event
        :return: None
        """
        if self.alive:
            keys = pygame.key.get_pressed()
            if_hot = check_hotkey(keys)
            if not if_hot:
                try:
                    self.input_text_list.append(KEYMAP[event.key])
                except KeyError as e:
                    self.process_special_key(event.key)
            else:
                if keys[pygame.K_LSHIFT] or keys[pygame.K_RSHIFT]:
                    self.process_shift_hotkey(keys)
                    if event.key in UperCheck.keys():
                        self.input_text_list.append(UperCheck[event.key])
                elif keys[pygame.K_LCTRL] or keys[pygame.K_RCTRL]:
                    self.process_ctrl_hotkey(event.key)
                elif keys[pygame.K_LALT] or keys[pygame.K_RALT]:
                    self.process_alt_hotkey(event.key)

    def check_key_up(self, event, window):
        """
        检测按键抬起
        :param event: Event
        :return: None
        """
        ...

    def process_special_key(self, key):
        """
        处理热键的函数
        :param key:键
        :return: None
        """
        if key == pygame.K_RETURN or key == pygame.K_KP_ENTER:
            self.alive = False
            self.input_img = self.formal_input_image
        elif key == pygame.K_BACKSPACE:
            self.delete_str()
        else:
            ...

    def delete_str(self):
        """
        删除字符的函数
        :return:
        """
        self.input_text_list = self.input_text_list[:-1]


class Input(Widget):
    def __init__(self,
                 size,
                 IDname,
                 theme,
                 font,
                 text_color=(255, 255, 255),
                 text_background=None,
                 cursor_color=(255, 0, 0),
                 password=False,
                 password_char="*"):
        """

        :param size: [int,int]
        :param IDname: str
        :param theme: Theme
        :param font: Font
        :param text_color:[int,int,int]
        :param text_background: [int,int,int]
        :param cursor_color: [int,int,int]
        """
        super().__init__(size, IDname)
        self.finish_call = None
        self.change_text_func = None
        self.password = password
        self.password_char = password_char
        self.up_function = None
        self.call_function = None
        self.text_color = text_color
        self.text_background = text_background
        self.cursor_color = cursor_color
        self.size = size
        self.pos = [0, 0]
        self.font = font

        self.formal_input_image = pygame.transform.scale(theme.get_theme()["formal_input"], self.size)
        self.chosen_input_image = pygame.transform.scale(theme.get_theme()["chosen_input"], self.size)

        self.input_img = self.formal_input_image

        self.input_text_list = []

        self.alive = False

        self.hot_key_dict = {}

        self.cursor_pos = [0, 0]
        self.cursor_if_blit = False
        self.event_types = [MOUSEDOWN, MOUSEUP, EVERYTIME, KEYDOWN]

    def return_input_text(self):
        """
        返回现在有的文字
        :return: None
        """
        str_text = ""
        for i in self.input_text_list:
            str_text += i
        return str_text

    def clear_text(self):
        """
        清空文字
        :return:None
        """
        self.input_text_list = []
        self.cursor_pos = self.pos

    def _compute_cursor_pos(self, rect):
        """
        重新计算光标坐标
        :param rect:
        :return:
        """
        self.cursor_pos = [self.pos[0] + rect[2], self.pos[1]]

    def bind_pos(self, new_pos: list):
        """
        绑定坐标
        :param new_pos:[]
        :return:
        """
        self.pos = new_pos
        self.cursor_pos = new_pos

    def _blit_cursor(self, screen):
        """
        光标绘画
        :return:NONE
        """
        if self.cursor_if_blit and self.alive:
            start_pos = [self.cursor_pos[0], self.cursor_pos[1] + 5]
            end_pos = [self.cursor_pos[0], self.cursor_pos[1] + self.size[1] - 5]
            pygame.draw.line(screen, self.cursor_color, start_pos, end_pos)

    def _change_cursor(self):
        """
        被当做任务调用！
        :return: None
        """
        self.cursor_if_blit = not self.cursor_if_blit

    def render_text(self):
        """
        递归判断是否当前文字长度超出范围
        :return:
        """
        if not self.password:
            text = ""
            for i in self.input_text_list:
                text += i
            text_surface = self.font.render_text(text, self.text_color, background=self.text_background)
            self._compute_cursor_pos(text_surface.get_rect())
            return text_surface
        else:
            text = self.password_char * len(self.input_text_list)
            text_surface = self.font.render_text(text, self.text_color, background=self.text_background)
            self._compute_cursor_pos(text_surface.get_rect())
            return text_surface

    def blit(self, window):
        """
        绘图
        :param window:
        :return:
        """
        screen = window.get_window()
        screen.blit(self.input_img, self.pos)
        self._blit_cursor(screen)
        if len(self.input_text_list):
            screen.blit(self.render_text(),
                        [self.pos[0], self.pos[1] + int((self.size[1] - int(self.font.font_size)) / 2)])

    def check_mouse_down(self, event, window):
        """
        检测鼠标按下并且调用回调函数的函数（笑）
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.size[0] and self.pos[1] < mouse_click_pos[1] < \
                self.pos[1] + self.size[1]:
            self.input_img = self.chosen_input_image
            self.alive = True
            if self.call_function:
                self.call_function(self)
        else:
            self.alive = False

    def check_mouse_left_down(self, event, window):
        """
        检测鼠标按下并且调用回调函数的函数（笑）
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.size[0] and self.pos[1] < mouse_click_pos[1] < \
                self.pos[1] + self.size[1]:
            self.input_img = self.chosen_input_image
            self.alive = True
            if self.call_function:
                self.call_function(self)
        else:
            self.alive = False

    def check_mouse_left_up(self, event, window):
        """
        检测鼠标按下并且调用回调函数的函数（笑）
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.size[0] and self.pos[1] < mouse_click_pos[1] < \
                self.pos[1] + self.size[1]:
            self.input_img = self.chosen_input_image
            self.alive = True
            if self.up_function:
                self.up_function(self)
        else:
            self.input_img = self.formal_input_image
            self.alive = False

    def check_mouse_up(self, event, window):
        """
        检测鼠标抬起并且调用回调函数的函数（笑）
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.size[0] and self.pos[1] < mouse_click_pos[1] < \
                self.pos[1] + self.size[1]:
            self.input_img = self.chosen_input_image
            self.alive = True
            if self.up_function:
                self.up_function(self)
        else:
            self.input_img = self.formal_input_image
            self.alive = False

    def register_for_event(self, window):
        """
        注册函数，将目标函数注册到EventProcessor中
        :param window: window对象
        :return: None
        """
        # window.event_processor.register_event(MOUSEDOWN, self.check_mouse_down)
        window.event_processor.register_event(MOUSE_BUTTON_LEFT_DOWN, self.check_mouse_left_down)
        window.event_processor.register_event(MOUSE_BUTTON_LEFT_UP, self.check_mouse_left_up)
        window.event_processor.register_event(EVERYTIME, self.weight_everytime_event)
        window.event_processor.register_event(KEYDOWN, self.check_key_down)
        window.event_processor.register_event(TEXTINPUT, self.check_text_input)
        window.register_for_task(INFINITE, self._change_cursor, 0.5)

    def process_ctrl_hotkey(self, key):
        """
        处理ctrl的快捷键
        :param key:
        :return:
        """
        if key == pygame.K_c:
            print("这里执行复制的操作....")
        elif key == pygame.K_v:
            print("这里执行粘贴的操作.....")
        elif key == pygame.K_x:
            print("这里执行剪切的操作.......")
        elif key == pygame.K_a:
            print("这里执行全选的操作.........")
        elif key == pygame.K_b:
            print("这里执行什么也不知道的操作....(只是cemeye想要炫耀一下这个可以随意定义快捷键的功能啦~)")

        # 这个是用来执行自定义的快捷键的（仅限有关Ctrl的）
        for i in self.hot_key_dict.keys():
            if pygame.K_LCTRL in i or pygame.K_RCTRL in i:
                if key in i:
                    self.hot_key_dict[i]()

    def process_alt_hotkey(self, key):
        """
        处理alt的快捷键
        :param key:
        :return:
        """
        # 这个是用来执行自定义的快捷键的（仅限有关alt的）
        for i in self.hot_key_dict.keys():
            if pygame.K_LALT in i or pygame.K_RALT in i:
                if key in i:
                    self.hot_key_dict[i]()

    def process_shift_hotkey(self, key):
        """
        处理shift的快捷键
        :param key:
        :return:
        """
        # 这个是用来执行自定义的快捷键的（仅限有关shift的）
        for i in self.hot_key_dict.keys():
            if pygame.K_LSHIFT in i or pygame.K_RSHIFT in i:
                if key in i:
                    self.hot_key_dict[i]()

    def check_key_down(self, event, window):
        """
        检测是否有按键按下
        :param event: Event
        :return: None
        """
        if self.alive:
            keys = pygame.key.get_pressed()
            if_hot = check_hotkey(keys)
            self.process_special_key(event.key)
            if if_hot:
                if keys[pygame.K_LSHIFT] or keys[pygame.K_RSHIFT]:
                    self.process_shift_hotkey(keys)

                elif keys[pygame.K_LCTRL] or keys[pygame.K_RCTRL]:
                    self.process_ctrl_hotkey(event.key)
                elif keys[pygame.K_LALT] or keys[pygame.K_RALT]:
                    self.process_alt_hotkey(event.key)

    def check_text_input(self, event, window):
        if self.alive:
            for i in event.text:
                self.input_text_list.append(i)
            if self.change_text_func:
                self.change_text_func()
            self.render_text()

    def process_special_key(self, key):
        """
        处理热键的函数
        :param key:键
        :return: None
        """
        if key == pygame.K_RETURN or key == pygame.K_KP_ENTER:
            self.alive = False
            self.input_img = self.formal_input_image
            if self.finish_call:
                self.finish_call()
        elif key == pygame.K_BACKSPACE:
            self.delete_str()
        else:
            ...

    def delete_str(self):
        """
        删除字符的函数
        :return:None
        """
        self.input_text_list = self.input_text_list[:-1]
        if len(self.input_text_list) == 0:
            self.cursor_pos = self.pos

    def change_to_password_type(self, password_char="*"):
        self.password_char = password_char
        self.password = True

    def change_to_formal_type(self):
        self.password = False

    def bind_change_text_func(self, func):
        self.change_text_func = func

    def bind_finish_call(self, func):
        """
        绑定回调函数
        :param func:func
        :return: None
        """
        self.finish_call = func



from SqrtS.Widgets.Widget import Widget
from SqrtS.Core.event_process import DROPFILE, EVERYTIME
import pygame


class FileDropper(Widget):
    """
    文件拖拽
    """

    def __init__(self, size, IDname, theme):
        super().__init__(size, IDname)
        self.pos = []
        self.file_dropper_img = pygame.transform.scale(theme.get_theme()["file_dropper"], size)
        self.cursor_in = False

        self.drop_files = []

    def bind_pos(self, new_pos: list):
        """
        绑定坐标
        :param new_pos: [int,int]
        :return: None
        """
        self.pos = new_pos

    def blit(self, window):
        """
        画出控件
        :param window: Window
        :return: None
        """
        window.get_window().blit(self.file_dropper_img, self.pos)

    def weight_everytime_event(self, window):
        """
        实时事件
        :param window:Window
        :return: None
        """
        mouse_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_pos[0] < self.pos[0] + self.size[0] and self.pos[1] < mouse_pos[1] < \
                self.pos[1] + self.size[1]:
            self.cursor_in = True
        else:
            self.cursor_in = False

    def check_drop_file(self, event, window):
        """
        检测拖拽文件的函数
        :param event: 事件
        :param window: Window
        :return: None
        """
        if self.cursor_in:
            self.drop_files.append(event.file)

    def register_for_event(self, window):
        """
        注册事件
        :param window: Window
        :return: None
        """
        window.event_processor.register_event(DROPFILE, self.check_drop_file)
        window.event_processor.register_event(EVERYTIME, self.weight_everytime_event)

    def get_last_file(self):
        """
        获取最后一个文件
        :return: str
        """
        if len(self.drop_files):
            return self.drop_files[-1]
        else:
            return None

    def get_all_file(self):
        """
        获取所有的文件
        :return: None
        """
        return self.drop_files



from abc import abstractmethod  # 抽象方法装饰器，声明一个必须被实现的方法
from SqrtS.Widgets.Widget import Widget


class Layout:
    """
    所有布局的基础类
    """

    def __init__(self, name: str):
        self.name = name

    @abstractmethod
    def compute_poses(self):
        """
        计算更新坐标
        :return: None
        """
        ...

    @abstractmethod
    def blit_all(self, window):
        """
        在屏幕上画出所有的控件
        :return: None
        """
        ...

    @abstractmethod
    def append(self, weight: Widget):
        ...

    @abstractmethod
    def appends(self, weights: list):
        ...



import math
from SqrtS.Core.TaskSystem import TIMES


class Vector:
    def __init__(self, x, y):
        self.x = x
        self.y = y

    def __add__(self, other):
        """
        加法
        :param other:
        :return:
        """
        return Vector(self.x + other.x, self.y + other.y)

    def __sub__(self, other):
        """
        减法
        :param other:
        :return:
        """
        return Vector(self.x - other.x, self.y - other.y)

    def __str__(self):
        """
        输出
        :return:
        """
        return f"Vector({self.x},{self.y})"

    def __mul__(self, other):
        """
        乘法
        :param other:
        :return:
        """
        return self.x * other.x + self.y * other.y

    def __iadd__(self, other):
        """
        自增
        :param other:
        :return:
        """
        self.x += other.x
        self.y += other.y
        return self

    def __isub__(self, other):
        """
        自减
        :param other:
        :return:
        """
        self.x -= other.x
        self.y -= other.y
        return self

    def __pos__(self):
        """
        当取+号时的操作
        :return:
        """
        return self

    def __neg__(self):
        """
        当取-号的时候
        :return:
        """
        return Vector(-self.x, -self.y)

    def __abs__(self):
        """
        取模长的操作
        :return:
        """
        return math.sqrt(self.x ** 2 + self.y ** 2)

    def __int__(self):
        """
        int方法，返回模长的整数
        :return:
        """
        return int(self.__abs__())

    def __getitem__(self, item):
        """
        可选的[0],[1],返回x或者y
        :param item:
        :return:
        """
        if item == 0:
            return self.x
        elif item == 1:
            return self.y

    def __setitem__(self, key, value):
        """
        设置x或y，[0],[1]
        :param key:
        :param value:
        :return:
        """
        if key == 0:
            self.x = int(value)
        elif key == 1:
            self.y = int(value)

    def __copy__(self):
        """
        获取列表格式的向量
        :return:list
        """
        return [self.x, self.y]


if __name__ == "__main__":
    v1 = Vector(1, 1)
    v2 = Vector(2, 2)
    for i in range(100):
        v2 -= v1
        print(v2)



from SqrtS.Animation.Animation import Animation, LowerAnimation
from SqrtS.Core.TaskSystem import TERMINABLE_INFINITE


class LoopAnimation(Animation):
    """
    动画基础类
    """

    def __init__(self, IDname):
        super().__init__(IDname)

    def times_blit_task(self):
        if self.img_index + 1 < len(self.surfaces_dict):
            self.img_index += 1
            self.now_img = self.surfaces_dict[str(self.img_index)]
        else:
            self.img_index = 0
            self.now_img = self.surfaces_dict[str(self.img_index)]

    def play(self, window, pos):
        self.pos = pos
        self.now_img = self.surfaces_dict[str(self.img_index)]
        if not self.had_been_play:
            window.register_for_task(TERMINABLE_INFINITE,
                                     self.times_blit_task,
                                     time_interval=self.setting["speed"],
                                     task_name=self.IDname)
            window.register_for_animation(self)
        self.had_been_play = True

    def stop(self, window):
        self.alive = False
        window.task_system.destroy_terminable_infinite_task(self.IDname)

    def pause(self, window):
        window.task_system.destroy_terminable_infinite_task(self.IDname)

    def unpause(self, window, pos):
        self.had_been_play = False
        self.play(window, pos)

    def blit_flip(self, window):
        if self.alive:
            window.get_window().blit(self.now_img, self.pos)
        else:
            window.destroy_animation(animation=self)


class LowerLoopAnimation(LowerAnimation):
    def __init__(self, IDname):
        super().__init__(IDname)

    def times_blit_task(self):
        if self.img_index + 1 < len(self.surfaces_dict):
            self.img_index += 1
            self.now_img = self.surfaces_dict[str(self.img_index)]
        else:
            self.img_index = 0
            self.now_img = self.surfaces_dict[str(self.img_index)]

    def play(self, window, pos, speed):
        self.pos = pos
        self.now_img = self.surfaces_dict[str(self.img_index)]
        if not self.had_been_play:
            window.register_for_task(TERMINABLE_INFINITE,
                                     self.times_blit_task,
                                     time_interval=speed,
                                     task_name=self.IDname)
            window.register_for_animation(self)
        self.had_been_play = True

    def stop(self, window):
        self.alive = False
        window.task_system.destroy_terminable_infinite_task(self.IDname)

    def pause(self, window):
        window.task_system.destroy_terminable_infinite_task(self.IDname)

    def unpause(self, window, pos, speed):
        self.had_been_play = False
        self.play(window, pos, speed)

    def blit_flip(self, window):
        if self.alive:
            window.get_window().blit(self.now_img, self.pos)
        else:
            window.destroy_animation(animation=self)



import pygame
from SqrtS.Widgets.Widget import Widget
from SqrtS.Widgets.Button import ImageTextButton
from SqrtS.Core.event_process import MOUSEUP, MOUSEDOWN, EVERYTIME


class ListBox(Widget):
    def __init__(self, size,
                 IDname,
                 theme,
                 font,
                 style=None,
                 choose_color=(255, 0, 0),
                 background=None
                 ):
        """

        :param size: [int,int]
        :param IDname: str
        :param theme: Theme
        :param font: Font
        :param style: [(),()]
        :param choose_color:[int,int,int]
        :param background: [int,int,int]
        """
        super().__init__(size, IDname)

        self.if_this_time_call_on = None
        self.chosen = None

        self.move_out_function = None
        self.move_on_function = None
        self.up_function = None
        self.call_function = None

        self.event_types = [MOUSEDOWN, MOUSEUP, EVERYTIME]

        self.pos = []
        self.listbox_img_formal = pygame.transform.scale(theme.get_theme()["listbox"], size)
        self.list_box_img_chosen = pygame.transform.scale(theme.get_theme()["listbox_chosen"], size)
        self.font = font
        self.style = style

        self.list_choosing = False

        self.listbox_img_now = self.listbox_img_formal

        self.list_item = []

        self.choose_color = choose_color
        self.background = background

        self.final_choose = ""
        self.final_choose_sur = self.font.render_text(text=self.final_choose,
                                                      color=self.choose_color,
                                                      background=self.background)
        self.text_pos = []

    def _render_choose(self):
        """
        渲染选择
        :return: None
        """
        self.final_choose_sur = self.font.render_text(text=self.final_choose,
                                                      color=self.choose_color,
                                                      background=self.background)
        self._compute_text_pos(self.final_choose_sur.get_rect())

    def _compute_text_pos(self, text_rect):
        """
        计算新的文字坐标
        :param text_rect:文字对象surface
        :return: 坐标列表
        """
        x = int((self.size[0] / 2) - (text_rect[2] / 2))
        y = int((self.size[1] / 2) - (text_rect[3] / 2))
        self.text_pos = [self.pos[0] + x, self.pos[1] + y]

    def _choose_update(self, args):
        """
        绑定给最终选择的函数，被按钮调用当做回调函数
        :param args: None
        :return: None
        """
        self.final_choose = args.IDname
        self._render_choose()

    def append_item(self, item_text):
        """
        初始时调用，即在未调用register函数前
        :param item_text: str
        :return: None
        """
        btn = ImageTextButton(button_size=self.size,
                              image_list=[self.listbox_img_formal,
                                          self.listbox_img_formal,
                                          self.list_box_img_chosen],
                              IDname=f"{item_text}",
                              button_text=item_text,
                              font=self.font,
                              style=self.style)
        btn.bind_call_func(self._choose_update)
        self.list_item.append(btn)

        self._compute_items_poses()

    def append_items(self, item_texts):
        """
        初始时调用，即在未调用register函数前
        :param item_texts: list
        :return: None
        """
        for i in item_texts:
            self.append_item(i)

    def add_items(self, item_texts, window):
        """
        任何时候追加选型都可以！
        :param item_texts: list
        :param window: Window
        :return: None
        """
        for item_text in item_texts:
            btn = ImageTextButton(button_size=self.size,
                                  image_list=[self.listbox_img_formal,
                                              self.listbox_img_formal,
                                              self.list_box_img_chosen],
                                  IDname=f"{item_text}",
                                  button_text=item_text,
                                  font=self.font,
                                  style=self.style)
            btn.bind_call_func(self._choose_update)
            self.list_item.append(btn)
            self._compute_items_poses()
            btn.register_for_event(window)

    def _compute_items_poses(self):
        """
        计算选项坐标
        :return: None
        """
        index = 1
        for i in self.list_item:
            i.bind_pos([self.pos[0], self.pos[1] + self.size[1] * index])
            index += 1

    def bind_pos(self, new_pos: list):
        """
        绑定坐标
        :param new_pos: list
        :return: None
        """
        self.pos = new_pos

    def blit_text(self, window):
        """
        画出文字
        :param window: Window
        :return: None
        """
        if self.final_choose:
            window.get_window().blit(self.final_choose_sur, self.text_pos)

    def blit(self, window):
        """
        画图
        :param window: Window
        :return: None
        """
        window.get_window().blit(self.listbox_img_now, self.pos)
        self.blit_text(window)
        if self.list_choosing:
            for i in self.list_item:
                i.blit(window=window)

    def check_mouse_down(self, event, window):
        """
        检测鼠标按下并且调用回调函数的函数（笑）
        :param window:
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.size[0] and self.pos[1] < mouse_click_pos[1] < \
                self.pos[1] + self.size[1]:
            self.list_choosing = True

            if self.call_function:
                self.call_function(self)
        else:
            self.list_choosing = False

    def check_mouse_up(self, event, window):
        """
        检测鼠标抬起并且调用回调函数的函数（笑）
        :param window:
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.size[0] and self.pos[1] < mouse_click_pos[1] < \
                self.pos[1] + self.size[1]:
            if self.up_function:
                self.up_function(self)
        else:
            self.list_choosing = False

    def weight_everytime_event(self, window):
        """
        每时每刻都在运行的函数为everytime函数，按钮内用于处理按钮移动上去的事件
        :return: None
        """
        mouse_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_pos[0] < self.pos[0] + self.size[0] and self.pos[1] < mouse_pos[1] < \
                self.pos[1] + self.size[1] and not self.list_choosing:
            self.listbox_img_now = self.list_box_img_chosen
            self.chosen = True

            if self.move_on_function and not self.if_this_time_call_on:
                self.move_on_function(self)
                self.if_this_time_call_on = True

        elif self.chosen:
            if self.move_out_function:
                self.move_out_function(self)

            self.chosen = False
            self.if_this_time_call_on = False

        else:
            self.listbox_img_now = self.listbox_img_formal

    def register_for_event(self, window):
        for i in self.list_item:
            i.register_for_event(window)
        """
        注册函数，将目标函数注册到EventProcessor中
        :param window: window对象
        :return: None
        """
        window.event_processor.register_event(MOUSEDOWN, self.check_mouse_down)
        window.event_processor.register_event(MOUSEUP, self.check_mouse_up)
        window.event_processor.register_event(EVERYTIME, self.weight_everytime_event)

    def bind_call_func(self, func):
        """
        绑定回调函数
        :param func:函数体
        :return: None
        """
        self.call_function = func

    def bind_up_func(self, func):
        """
        绑定结束函数
        :param func:函数体
        :return: None
        """
        self.up_function = func

    def bind_on_func(self, func):
        """
        绑定鼠标一上去的函数
        :param func:函数体
        :return: None
        """
        self.move_on_function = func

    def bind_out_func(self, func):
        """
        绑定鼠标一出去的函数
        :param func:函数体
        :return: None
        """
        self.move_out_function = func

    def get_final_choose(self):
        """
        获取最终选择，str
        :return: None
        """
        return self.final_choose

    def clear_items(self, window):
        self.list_item = []
        self.final_choose = ""
        self.final_choose_sur = self.font.render_text(text=self.final_choose,
                                                      color=self.choose_color,
                                                      background=self.background)
        self.list_choosing = False

        for i in self.list_item:
            i.destroy_widget(window=window)

    def delete_item(self, index, window):
        if len(self.list_item):
            self.list_item[index].destroy_widget(window=window)
            self.list_item.remove(self.list_item[index])
            self.list_choosing = False



from SqrtS.Core.event_process import MOUSEDOWN, MOUSEUP, EVERYTIME
from SqrtS.Widgets.Widget import Widget
import pygame

"""
拓展模块之一：拖拽板
可以检测拖拽并且调用相应的函数！
"""


class DraggingBoard(Widget):
    """
    拖拽板
    """

    def __init__(self, size, IDname):
        """
        :param size:[int,int]
        :param IDname: str
        """
        super().__init__(size, IDname)
        self.pos = []
        self.if_mouse_down = False
        self.last_mouse_pos = []

        self.call_slide_function = None

    def bind_pos(self, new_pos: list):
        """
        固定绑定坐标
        :param new_pos:[]
        :return: None
        """
        self.pos = new_pos

    def blit(self, window):
        """
        拖拽板哪里有绘图函数?为了直接兼容，所以只能写这个函数
        :param window: None
        :return: None
        """
        ...

    def bind_call_slide(self, func):
        """
        绑定滑动函数
        :param func:函数体
        :return: None
        """
        self.call_slide_function = func

    def check_mouse_down(self, event, window):
        """
        检测鼠标按下并且调用回调函数的函数（笑）
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.size[0] and self.pos[1] < mouse_click_pos[1] < \
                self.pos[1] + self.size[1]:
            self.if_mouse_down = True
            self.last_mouse_pos = pygame.mouse.get_pos()
        else:
            self.if_mouse_down = False

    def check_mouse_up(self, event, window):
        """
        检测鼠标弹起并且调用回调函数的函数（笑）
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.size[0] and self.pos[1] < mouse_click_pos[1] < \
                self.pos[1] + self.size[1]:
            self.if_mouse_down = False
        else:
            self.if_mouse_down = False

    def weight_everytime_event(self, window):
        """
        内部函数的实时函数
        :return: None
        """
        mouse_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_pos[0] < self.pos[0] + self.size[0] and self.pos[1] < mouse_pos[1] < \
                self.pos[1] + self.size[1]:
            if self.if_mouse_down and mouse_pos != self.last_mouse_pos:
                self.call_slide_function()
        else:
            ...

    def register_for_event(self, window):
        """
        注册函数
        :param window: Window
        :return: None
        """
        window.event_processor.register_event(MOUSEDOWN, self.check_mouse_down)
        window.event_processor.register_event(MOUSEUP, self.check_mouse_up)
        window.event_processor.register_event(EVERYTIME, self.weight_everytime_event)


class GlobalDraggingBoard(DraggingBoard):
    """
    全局拖拽板（☞可以获取全局坐标的改变）
    """
    def __init__(self, size, IDname):
        super().__init__(size, IDname)
        self.window_pos = []
        self.abs_pos = []

    def update_info(self, window):
        """
        更新信息，内部函数
        :param window:
        :return:
        """
        self.window_pos = window.window_mover.get_window_pos()

    def check_pos_in(self, pos_click):
        """
        检测坐标是否在控件内部
        :param pos_click: 需要判断的坐标[int, int]
        :return: bool
        """
        if self.pos[0] < pos_click[0] < self.pos[0] + self.size[0] and self.pos[1] < pos_click[1] < \
                self.pos[1] + self.size[1]:
            return True
        else:
            return False

    def check_pos_in_abs(self, pos_click):
        """
        检测绝对坐标是否在控件内
        :param pos_click: 坐标[int, int]
        :return: None
        """
        if self.abs_pos[0] < pos_click[0] < self.abs_pos[0] + self.size[0] and self.abs_pos[1] < pos_click[1] < \
                self.abs_pos[1] + self.size[1]:
            return True
        else:
            return False

    def check_mouse_down(self, event, window):
        """
        检测鼠标按下并且调用回调函数的函数（笑）
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        abs_mouse_pos = [mouse_click_pos[0] + self.window_pos[0],
                         mouse_click_pos[1] + self.window_pos[1]]

        self.abs_pos = [
            self.pos[0] + self.window_pos[0],
            self.pos[1] + self.window_pos[1]
        ]

        if self.check_pos_in_abs(abs_mouse_pos):
            self.if_mouse_down = True
            self.last_mouse_pos = pygame.mouse.get_pos()
        else:
            self.if_mouse_down = False

    def check_mouse_up(self, event, window):
        """
        检测鼠标弹起并且调用回调函数的函数（笑）
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        abs_mouse_pos = [mouse_click_pos[0] + self.window_pos[0],
                         mouse_click_pos[1] + self.window_pos[1]]
        self.abs_pos = [
            self.pos[0] + self.window_pos[0],
            self.pos[1] + self.window_pos[1]
        ]

        if self.check_pos_in_abs(abs_mouse_pos):
            self.if_mouse_down = False
        else:
            self.if_mouse_down = False

    def weight_everytime_event(self, window):
        """
        注册实时事件
        :return:
        """
        if self.if_mouse_down:
            self.call_slide_function()
        else:
            ...






from SqrtS.Core.core import Window, InferiorWindow
from SqrtS.tools.image import ImageSurface, SurfaceImage
from SqrtS.tools.load_theme import Theme
from SqrtS.Widgets.Button import ImageButton, Button
from SqrtS.Widgets.Text import Texts, LineTexts
from SqrtS.Layouts.FloatingLayout import FloatingLayout


OK_CANCEL = 1
OK = 2
TRUE_FALSE = 3


class PopupWindow:
    """
    弹窗基类
    """

    def __init__(self, popup_size,
                 theme: Theme,
                 title,
                 text,
                 text_line_length,
                 font,
                 popup_image_path=False,
                 close_button_size=None,
                 style=OK_CANCEL,
                 title_style=None,
                 texts_style=None):
        """

        :param popup_size: [int,int]
        :param theme: Theme
        :param title: str
        :param text: str
        :param text_line_length:int
        :param font: Font
        :param popup_image_path:str
        :param close_button_size: [int,int]
        :param style: [(),()]
        :param title_style: [(),()]
        :param texts_style: [(),()]
        """
        if texts_style is None:
            texts_style = [[0, 0, 0], None]
        if title_style is None:
            title_style = [[0, 0, 0], None]
        # 定义文字和标题对象
        self.title_sur = Texts(title, title_style[0], font, "title", True, title_style[1])
        self.texts = LineTexts(text, texts_style[0], font, text_line_length, "texts", True, texts_style[1])
        self.title_pos = []
        self.texts_pos = []
        # 定义按钮样式
        self.style = style
        # 按钮大小
        if close_button_size is None:
            close_button_size = [20, 20]
        # 弹窗大小
        self.popup_size = popup_size
        # 窗口状态
        self.alive = False
        # 窗口固定坐标
        self.popup_back_pos = [0, 0]
        # 关闭按钮参数
        self.close_button_size = close_button_size
        self.image_close_list = [
            theme.get_theme()["kill"],
            theme.get_theme()["kill_click"],
            theme.get_theme()["kill_chosen"]
        ]
        self.close_button = ImageButton(self.close_button_size, self.image_close_list, IDname="popup_close_button")
        self.close_button_pos = [0, 0]
        # 定义布局
        self.root_layout = FloatingLayout(name="root_popup")
        # 弹窗背景图片
        if not popup_image_path:
            self.popup_back_image = SurfaceImage(theme.get_theme()["popup"]).resize(self.popup_size)
        else:
            self.popup_back_image = ImageSurface(popup_image_path).resize(popup_size)
        # 弹窗返回值
        self.return_value = ""

    def develop_other_widgets(self, theme, font, window, popup_window):
        """

        :param theme: Theme
        :param font: Font
        :param window: Window
        :param popup_window: PopupWindow
        :return: None
        """
        self._compute_popup_pos(window_size=window.get_window_size())
        self.root_layout.append(self.close_button)
        self.root_layout.bind_widgets_poses({"popup_close_button": self.close_button_pos})
        self.root_layout.append(self.title_sur)
        self.root_layout.append(self.texts)

        if self.style == OK_CANCEL:
            def call_ok(args):
                self.return_value = "ok"
                self.alive = False
                popup_window.alive = False

            def call_cancel(args):
                self.return_value = "cancel"
                self.alive = False
                popup_window.alive = False

            ok_btn = Button(
                [100, 40], "OK", theme, font, "ok_btn"
            )
            self.root_layout.append(ok_btn)
            ok_btn.register_for_event(popup_window)
            ok_btn.bind_call_func(call_ok)
            # --------
            cancel_btn = Button(
                [100, 40], "Cancel", theme, font, "cancel_btn"
            )
            self.root_layout.append(cancel_btn)
            cancel_btn.register_for_event(popup_window)
            cancel_btn.bind_call_func(call_cancel)
            # --------
            self.root_layout.bind_widgets_poses(widgets_pos_dict={
                "ok_btn": [
                    self.popup_back_pos[0] + self.popup_size[0] - 220, self.popup_back_pos[1] + self.popup_size[1] - 50
                ], "cancel_btn": [
                    self.popup_back_pos[0] + self.popup_size[0] - 110, self.popup_back_pos[1] + self.popup_size[1] - 50
                ]
            })
        elif self.style == OK:
            def call_ok(args):
                self.return_value = "ok"
                self.alive = False
                popup_window.alive = False

            ok_btn = Button(
                [100, 40], "OK", theme, font, "ok_btn"
            )
            self.root_layout.append(ok_btn)
            ok_btn.register_for_event(popup_window)
            ok_btn.bind_call_func(call_ok)
            # --------
            self.root_layout.bind_widgets_poses(widgets_pos_dict={
                "ok_btn": [
                    self.popup_back_pos[0] + self.popup_size[0] - 110, self.popup_back_pos[1] + self.popup_size[1] - 50
                ],
            })

        elif self.style == TRUE_FALSE:
            def call_true(args):
                self.return_value = "True"
                self.alive = False
                popup_window.alive = False

            def call_false(args):
                self.return_value = "False"
                self.alive = False
                popup_window.alive = False

            true_btn = Button(
                [100, 40], "是", theme, font, "true_btn"
            )
            self.root_layout.append(true_btn)
            true_btn.register_for_event(popup_window)
            true_btn.bind_call_func(call_true)
            # --------
            false_btn = Button(
                [100, 40], "否", theme, font, "false_btn"
            )
            self.root_layout.append(false_btn)
            false_btn.register_for_event(popup_window)
            false_btn.bind_call_func(call_false)
            # --------
            self.root_layout.bind_widgets_poses(widgets_pos_dict={
                "true_btn": [
                    self.popup_back_pos[0] + self.popup_size[0] - 220, self.popup_back_pos[1] + self.popup_size[1] - 50
                ], "false_btn": [
                    self.popup_back_pos[0] + self.popup_size[0] - 110, self.popup_back_pos[1] + self.popup_size[1] - 50
                ]
            })

    def _compute_popup_pos(self, window_size):
        """
        重新计算背景图片的位置
        :param window_size:
        :return:
        """
        self.popup_back_pos = [
            int((window_size[0] - self.popup_size[0]) / 2),
            int((window_size[1] - self.popup_size[1]) / 2)

        ]
        self.close_button_pos = [
            self.popup_back_pos[0] + self.popup_size[0] - self.close_button_size[0],
            self.popup_back_pos[1],
        ]

        self.title_pos = [self.popup_back_pos[0] + 10, self.popup_back_pos[1] + 10]
        self.texts_pos = [
            self.popup_back_pos[0] + 30, self.popup_back_pos[1] + 50
        ]

        self.title_sur.bind_pos(self.title_pos)
        self.texts.bind_pos(self.texts_pos)
        self.close_button.bind_pos(self.close_button_pos)

    def start(self, window: Window, theme, font):
        """
        运行弹窗（注意！该控件（也许不是控件）是另开了一个循环！元循环中的所有GUI将被暂停！）
        :return: None
        """

        def close(args):
            self.alive = False
            popup_window.alive = False

        self.close_button.bind_call_func(close)

        self.alive = True

        popup_window = InferiorWindow(window)
        self.develop_other_widgets(theme, font, window, popup_window)
        popup_window.append_draw_thing(["popup_back", self.popup_back_image, self.popup_back_pos])
        popup_window.bind_layout(self.root_layout)

        self.close_button.register_for_event(popup_window)

        popup_window.run()



import pyautogui
from SqrtS.Expand.DraggingBoard import GlobalDraggingBoard
from SqrtS.Core.TaskSystem import ONCE
from SqrtS.Expand.WindowMove import WindowMover
from SqrtS.Widgets.Widget import Widget


class WindowDraggingBoard(Widget):
    """
    未经测试
    """
    def __init__(self, window_size, window, size, IDname):
        """

        :param window_size: [int,int]
        :param window: Window
        :param size: [int,int]
        :param IDname: str
        """
        super().__init__(size, IDname)
        self.pos = []
        self.offset_address = window.offset_address
        self.window_size = window_size
        self.dragging_board = GlobalDraggingBoard([self.window_size[0], self.offset_address], "_slide_board")
        self.dragging_board.register_for_event(window=window)
        self.dragging_board.bind_pos([0, 0])
        self.dragging_board.bind_call_slide(self._drag)

        self.window_mover = WindowMover()

    def bind_pos(self, new_pos: list):
        """
        绑定坐标
        :param new_pos:
        :return:
        """
        self.pos = new_pos

    def blit(self, window):
        ...

    def register_for_event(self, window):
        """
        注册时间
        :param window:
        :return:
        """
        window.register_for_task(type_=ONCE, func=self.dragging_board.update_info, time_interval=1, args=window)

    def _drag(self):
        """
        拖动
        :return:
        """
        abs_mouse_pos = pyautogui.position()
        pygame_pos = self.dragging_board.last_mouse_pos
        self.window_mover.moveto([abs_mouse_pos[0] - pygame_pos[0], abs_mouse_pos[1] - pygame_pos[1]])



from setuptools import setup
import setuptools

setup(
    name='SqrtS',
    version='0.0.1',
    author='cemeye',
    author_email='caomingyang2022@163.com',
    url='https://pypi.org/project/SqrtS',
    description='python magic GUI package',
    # 需要打包的目录，只有这些目录才能 from import
    packages=setuptools.find_packages(),
    package_data={
        '': ['*.zip'],
        'bandwidth_reporter': ['*.zip']
    },
    # 安装此包时需要同时安装的依赖包
    install_requires=['pywin32', "pillow", "pyautogui", "pygame", "keyboard"],
)



import win32api
import win32con
import win32gui
import pygame


def window_transparency(window):
    """
    窗口透明化函数
    :param window:Window
    :return: None
    """
    transparency_color = (20, 20, 20)  # Transparency color
    window.bind_background_color(transparency_color)
    hwnd = pygame.display.get_wm_info()["window"]
    win32gui.SetWindowLong(hwnd, win32con.GWL_EXSTYLE,
                           win32gui.GetWindowLong(hwnd, win32con.GWL_EXSTYLE) | win32con.WS_EX_LAYERED)
    win32gui.SetLayeredWindowAttributes(hwnd, win32api.RGB(*transparency_color), 0, win32con.LWA_COLORKEY)
